function fnsingg(nomfifs,affnfi,driver,param,typf,taicar)
%function fnsingg(nomfifs,affnfi,driver,param,typf,taicar)
%
% Representation graphique des fonctions singulieres angulaires
% ou de la partie angulaire des contraintes associees a chaque exposant
% de singularite.
% Creation d'une ou plusieurs figures ou sont regroupees les differentes
% fonctions presentes dans le fichier, eventuellement associees
% a differents exposants de singularites.
% Chaque figure comporte au maximum nlix fonctions car au dela, les
% graphiques sont trop serres pour figurer ensemble sur une meme page
% de dimensions A4.
%
% ENTREES
%  nomfifs : nom du fichier contenant les valeurs des fonctions
%  affnfi  : vaut 1 pour afficher le nom du fichier, 0 sinon
%  driver  : nom du driver pour creer un fichier de sortie (option de la
%            commande print) ; vaut '' si pas de sortie souhaitee.
%  param   : vaut 1 pour afficher le parametre de calcul, 0 sinon
%  typf   : typf(1) = type de fonction a tracer ; vaut
%             . 1 pour une fonction singuliere
%             . 2 pour les contraintes
%           typf(2) = type de coordonnees a utiliser pour tracer les deux
%                     premieres composantes de f ; vaut
%             . 1 pour les coordonnees cartesiennes (valeur par defaut)
%             . 2 pour les coordonnees polaires
%  taicar  : taille des caracteres (en points) pour les legendes

%
%  Initialisations
nlix = 7;
legtrait = notex('___ real  _ _ imag ');
if typf(1) == 1, legy = 'Sing. funct. '; else, legy = 'Stress funct. '; end
[v,d]=version;
if v(1) == '4'
 nutxt = 'nu';
else
 nutxt = '\nu';
end
%
%               ------------  ETAPE 1 ------------
%  Pour configurer correctement le dessin (parametres de subplot),
%  on parcourt un premiere fois le fichier en comptant le nombre
%  d'exposants a traiter et leur multiplicite.
[fifs, message] = fopen(nomfifs);
if (fifs < 0) error([message,' (fichier ',nomfifs,')']), end
point=min(find(nomfifs=='.'));
ext = nomfifs(point+1:length(nomfifs));
%
[omega,pts,nu,multi,dim,sseprc,parcou,nomfic,ok]=lpctrl(fifs);
nco = dim;
nli = [];
nlicou = 0;
while ok ~= 0
 if  nlicou+multi > nlix
  nli = [nli nlicou];
  nlicou = multi;
 else
  nlicou = nlicou+multi;
 end
 nbpts = sum(pts);
 for m=1:multi
  mat = fscanf(fifs,'%g',[2*dim+1 nbpts]);
 end
 [omega,pts,nu,multi,dimb,sseprc,parcb,nomficb,ok]=lpctrl(fifs);
end
fclose(fifs);
nli = [nli nlicou]; 
nbfig = length(nli);
%
%               ------------  ETAPE 2 ------------
%      Traitement des donnees et generation des graphes
%
[fifs, message] = fopen(nomfifs);
if (fifs < 0) error([message,' (fichier ',nomfifs,')']), end
[omega,pts,nu,multi,dimb,sseprc,parcb,nomficb,ok]=lpctrl(fifs);
%
for numfig = 1:nbfig
nlipag = nli(numfig);
figure
numdes = 0;
hax = zeros(nlipag,nco);
nlicou = 0;
racrea = 1;       % mis a 0 si un des exposants est complexe
%
while nlicou+multi <= nlipag
% Trace de la (des) fonction(s) singuliere(s) relative(s)
% a l'exposant nu suivant.
 nbpts = sum(pts);
%
% On teste si l'exposant de singularite est reel (a sseprc pres),
% auquel cas la partie imaginaire de la fonction singuliere est nulle
% et n'est pas visible sur le graphique.
 if (abs(imag(nu)) < sseprc)
  valnu = num2str(real(nu),7);
 else
  valnu = num2str(nu,7);
  racrea = 0;
 end
 titnu = [nutxt,' = ',valnu];
%
% if (nlicou > 0), set(hax(nlicou,dim-1),'XTick',[]), end
 if (nlicou > 0), set(hax(nlicou,:),'XTick',[]), end
 nlifn1 = nlicou + 1;
 for m=1:multi
  mat  = fscanf(fifs,'%g',[2*dim+1 nbpts]);
  t = mat(1,:);
  f = zeros(dim,size(mat,2));
  nlicou = nlicou+1;
  for k=1:dim
   f(k,:) = mat(2*k,:) + i*mat(2*k+1,:);
   numdes = numdes + 1;
   hax(nlicou,k) = subplot(nlipag,nco,numdes);
  end
  tr1fs(dim,t,f,omega,nu,[legy,int2str(m)],hax(nlicou,:),taicar,typf(2))
  if m ~= multi, set(hax(nlicou,:),'XTick',[]), end
 end

 nlitit = nlicou-multi+1;
 if param
  titpar = ['P = ',num2str(parcou,6)];
  if dim == 2, titnu = [titpar,', ',titnu]; end
  axes(hax(nlitit,1)), title(titpar)
 end
 axes(hax(nlitit,dim-1)), title(titnu)

% T='_' avec Matlab4, T='\_' avec Matlab5
% if (nlifn1 > 1 & separ)
%  set(get(hax(nlifn1,1),'Title'),'HorizontalAlignment','right',...
%      'String',[T,T,T,T,T,T,T,T,T,T,T,T,T,T,T])
%  set(get(hax(nlifn1,dim),'Title'),'HorizontalAlignment','left',...
%      'String',[T,T,T,T,T,T,T,T,T,T,T])
% end
 [omega,pts,nu,multi,dimb,sseprc,parcb,nomficb,ok]=lpctrl(fifs);
end % while
%
% Legende en x sur les graphiques inferieurs uniquement
if typf(2) == 2
 labtyc = [' (radial)';' (angul.)';'         '];
else
 labtyc = [' ';' ';' '];
end
for k=1:dim
 axes(hax(nlipag,k))
 xlabel(['Comp. ',int2str(k),labtyc(k,:)])
end
%
% Affichage des informations en tete du dessin
if (racrea)
 leginfo = ' ';
else
 leginfo = legtrait;
end
%
if affnfi, leginfo = [leginfo,notex(nomfic)]; end
axes(hax(1,dim))
title(leginfo)
%
%  Creation eventuelle du fichier Postscript
valnu = ['GR',int2str(numfig)];
printps(parcou,nomfic,ext,valnu,nlipag,driver)
end % for numfig
%
fclose(fifs);
