function [omega,pts,nu,multi,dim,sseprc,parcou,nomfic,ok]=lpctrl(fifs)
%function [omega,pts,nu,multi,dim,sseprc,parcou,nomfic,ok]=lpctrl(fifs)
%
% Lecture des parametres de controle dans le fichier associe au
% descripteur fifs.
%
% ENTREES
%   fifs   : descripteur du fichier de donnees
% SORTIES
%   omega  : angles (en radians) definissant la geometrie des domaines
%   pts    : nombres de points de la courbe dans chaque domaine
%   nu     : exposant de singularite
%   multi  : multiplicite de nu
%   dim    : dimension de l'espace
%   sseprc : seuil de separation des racines reelles et complexes
%   parcou : parametre courant pilotant le calcul
%   nomfic : nom du fichier de donnees
%   ok     : indicateur valant 0 si la fin de fichier est rencontree au
%            debut du groupe de parametres lus ; est different de 0 sinon

%     omega(0), nbdom
[ld ok] = fscanf(fifs,'%g %d',2);
%
%     Si la fin de fichier n'est pas atteinte, on continue la lecture.
if ok ~= 0
%
%     omega(i), nb points dans le domaine i
 ls = fscanf(fifs,'%g %d',[2 ld(2)]);
 omega = [ld(1) ls(1,:)];
 pts = ls(2,:);
%
%     real(nu), aimag(nu)
 ld = fscanf(fifs,'%g %g',2);
 nu = ld(1) + i*ld(2);
%
%     Multiplicite et dimension
 ld = fscanf(fifs,'%d %d',2);
 multi = ld(1);
 dim   = ld(2);
%
%     Seuil de separation des racines reelles et complexes
%     et parametre pilotant le calcul
 ld = fscanf(fifs,'%g %g',2);
 sseprc = ld(1);
 parcou = ld(2);
%
%     Nom du fichier de donnees
 nomfic = fscanf(fifs,'%s',1);
else
 omega=[];pts=[];nu=[];multi=[];dim=[];sseprc=[];parcou=[];nomfic=[];
end
