function modiax(haxes,limites)
% function modiax(haxes,limites)
%
%   Modification des limites des axes sur les graphiques representant
%  les exposants de singularites. Si tous les exposants sont reels,
%  il n'y qu'un graphique.
%   S'il y a 2 graphiques, les limites horizontales sont les memes.
%  Sur le graphique des parties imaginaires, les limites verticales sont
%  modifiees (min = 0) et la dimension verticale est adaptee de maniere
%  a obtenir la meme echelle que pour les parties reelles, sauf si les
%  parties relles sont trop "petites" (le cadre serait trop "ecrase").
%
%  haxes : handles des axes de la figure (partie reelle et imag.)
%  limites : vecteur [xmin xmax yminRe ymaxRe] si 1 graphique,
%		     [xmin xmax yminRe ymaxRe ymaxIm] si 2 graphiques.
%	     [xmin xmax yminRe ymaxRe] sont les dimensions imposees sur
%	     le graphique des parties reelles
%	     [xmin xmax    0   ymaxIm] sont les dimensions imposees sur
%	     le graphique des parties imaginaires

if length(limites) < 4 | (length(haxes) > 1 & length(limites) < 5)
 disp('*** Error while scaling axes : missing data.'), return
end
%    Partie reelle
if limites(1) >= limites(2) | limites(3) >= limites(4)
 disp('*** Error while scaling axes :')
 disp('Values of axes limits incompatible among themselves.'), return
end
haxr = haxes(1);
axes(haxr)
axis([limites(1) limites(2) limites(3) limites(4)])
%
%    Partie imaginaire
if length(haxes) > 1
if limites(5) <= 0
 disp('*** Error while scaling axes : YmaxIm must be > 0.'), return
end
 haxi = haxes(2);
 axes(haxi)
 ylimi=get(haxi,'Ylim');
% Le min est une precaution au cas ou ylim(1) serait < 0 (cas d'erreur
% que l'on pourrait alors detecter)
 ylimi(1)=min(0,ylimi(1));
 axis([limites(1) limites(2) ylimi(1) limites(5)])
%
% Si dy(haxi) <= dy(haxr), on impose une dimension verticale du cadre
% haxi proportionnelle a celle du cadre haxr, limitee inferieurement
% a la fraction frac de cette dimension.
 frac = 0.25;
 disdr = (limites(5)-ylimi(1)) / (limites(4)-limites(3));
 if disdr <= 1
  recti = get(haxi,'Position');
  rectr = get(haxr,'Position');
  recti(4) = max(frac,disdr)*rectr(4);
  set(haxi,'Position',recti);
 end
end
