function p = phian(theta,nu,alpha,qplus,dim)
%function p = phian(theta,nu,alpha,qplus,dim)
%
% Calcul de la base des fonctions singulieres dans le cas anisotrope
%
% ENTREES
%  theta  : coordonnee angulaire (abscisse d'evaluation en radians)
%  nu     : exposant de singularite
%  alpha  : racines du determinant du symbole M+
%  qplus  : vecteurs directeurs de la base correspondant a M+
%  dim    : dimension de l'espace
%
% SORTIES
%  p : matrice des 2*dim vecteurs de la base

epitn = exp(i * theta * nu);
emitn = 1 / epitn;
e2it = exp(2*i*theta);
p = zeros(dim,2*dim);
%
for j=1:dim
 jpd = j+dim;
 gamp1 = alpha(j) * e2it + 1;
 p(:,j)   = emitn * gamp1^nu * qplus(:,j);
 p(:,jpd) = epitn * conj(gamp1)^nu * conj(qplus(:,j));
end
