function repfs(contx,omega,nu,alpha,qplus,coef,nbpts,sseprc,parcou,nomfic,ext)
%function repfs(contx,omega,nu,alpha,qplus,coef,nbpts,sseprc,parcou,nomfic,ext)
%
% Representation graphique des fonctions singulieres angulaires
% associees a un exposant de singularite
%
% ENTREES
%  contx  : contexte de calcul pour chaque domaine (1=isotrope)
%  omega  : angles (en radians) definissant la geometrie des domaines
%  nu     : exposant de singularite
%  alpha  : racines du determinant du symbole M+
%  qplus  : vecteurs directeurs de la base correspondant a M+
%  coef   : coefficients de la combinaison lineaire definissant
%           la fonction singuliere (noyau de A(nu))
%  nbpts  : nombre de points de la courbe
%  sseprc : seuil de separation des racines reelles et complexes
%  parcou : parametre courant pilotant le calcul
%  nomfic : nom du fichier des donnees traitees
%  ext    : extension du nom du fichier de donnees

% Reglages locaux :
%  affnfi : vaut 1 pour afficher le nom du fichier, 0 sinon
%  driver : nom du driver pour creer un fichier de sortie (option de la
%           commande print) ; vaut '' si pas de sortie souhaitee.
%  titpar : titre concernant le parametre de calcul. Peut etre vide.
%  typf(1): type de fonction a tracer ; vaut 1 pour une fonction singuliere
%  typf(2): type de coordonnees (1 pour cartesiennes, 2 pour polaires)
%  taicar : taille des caracteres (en points) pour les legendes

affnfi = 1;
driver = '';
%driver = '-deps2';
titpar = ['P = ',num2str(parcou,6)];
typf = [1 2];
taicar = 12;
%
if affnfi, nfic = nomfic; else nfic = []; end
multi = size(coef,2);
figure
for j=1:multi
 [t f] = calfs(contx,omega,nu,alpha,qplus,coef(:,j),nbpts);
 valnu = trfs(t,f,omega,nu,multi,sseprc,j,nfic,titpar,typf,taicar);
end
printps(parcou,nomfic,ext,valnu,multi,driver)
