function uifsf(action)
%
%  Representation graphique des fonctions singulieres angulaires
%  ou des contraintes.
%  Fonction regroupant les differentes actions a realiser selon les
%  choix effectues via l'interface graphique conversationnelle.
%

switch action

% ========================
% Definition des CreateFcn
% ========================

% Recuperation du repertoire racine, generation du repertoire des donnees
% creation d'une figure attachee a la figure maitresse.
% -----------------------------------------------------------------------
case 'iniappli'
 ficdir = 'exsiel.dir';
 fid = fopen(ficdir);
 if fid > 0
  repdon = fscanf(fid,'%s');
 else
  repdon = cd;
  disp(['*** File ', ficdir, ' not found.'])
  disp('Looking for data files in the current directory')
  disp(repdon)
 end
 typf = get(gcbf,'UserData');
 posi = get(gcbf,'Position');
 if typf == 1
  titre = 'Angular singular functions';
 else
  titre = 'Angular stress functions';
  posi(2) = posi(2)-posi(4)*1.25;
 end
 set(gcbf,'UserData',[{[repdon,'/']},{typf}],'Name',titre,'Position',posi)

% Initialisation des drivers de sortie
% ------------------------------------
case 'inioutput'
% PopupMenuOutput
 lisdri = [{'None'},{'Postscript'},{'Color Postscript'}];
 option = [{''},{'-deps2'},{'-depsc2'}];
 set(gcbo,'String',lisdri,'UserData',option)

% Initialisation de l'affichage des tailles de caracteres
% -------------------------------------------------------
case 'initaicar'
% PopupMenuTaicar
 listai = [{'9'},{'10'},{'11'},{'12'},{'13'},{'14'},{'15'},{'16'}];
 set(gcbo,'String',listai,'Value',4)

% Initialisation de l'affichage du popmenu
% ----------------------------------------
case 'initmode'
% PopupMenuMode
 lismod = [{'Individually'},{'Grouped'}];
 set(gcbo,'String',lismod,'Value',1)
% uifsf mode

% Initialisation de l'affichage de la listbox contenant
% la liste des fichiers de donnees (d'extension .sgf ou
% .trf selon le type de fonction choisi)
% -----------------------------------------------------
case 'lisfidon'
% ListboxDonneesFs
 ud = get(gcbf,'UserData');
 repdon = ud{1};
 if ud{2} == 1, typfic = '*.sgf'; else typfic = '*.trf'; end
 strar = dir([repdon,typfic]);
 celar = struct2cell(strar);
%  On selectionne les fichiers, en ecartant les repertoires, puis
%  on trie par ordre alphabetique.
 if ~ isempty(strar)
  ind = find([strar.isdir] == 0);
 else
  ind = [];
 end
 if isempty(ind)
   disp('*** No data file found. ***')
 else
   set(gcbo,'String',cellstr(sortrows(char(celar(1,ind)))))
 end

% =======================
% Definition des Callback
% =======================
case 'fin'
% SubuimenuConfirm : fin de l'application, fermeture de la fenetre
 close(gcbf)

case 'help'
% uimenuHelp : aide sur cette application
 helptext=[
 ['                                                                               ']
 ['Data file   : Choose a data file by selecting an item.                         ']
 ['Indiv/Group : The data file may contain several functions, corresponding       ']
 ['              to different singular exponents. You can display them separately,']
 ['              one figure for each exponent, or in groups.                      ']
 ['Display options                                                                ']
 [' Filename   : Switches on and off the display of the data filename at the right']
 ['              top of the graph.                                                ']
 [' Parameter  : Switches on and off the display of the parameter governing the   ']
 ['              computation. It is displayed as "P = value". For instance, it can']
 ['              be the orientation angle of the fibers of a material.            ']
 ['                                                                               ']
 ['Output      : Sets the type of output file desired. Default is no output. In   ']
 ['              individual mode, a file is created for each exponent. In grouped ']
 ['              mode, several files may be created. The files are created in your']
 ['              output directory. Their names are given in the Matlab window.    ']
 ];
 disp(helptext)

%case 'mode'
%% PopupMenuMode : validation ou invalidation de la CheckboxSepa
% val = get(gcbo,'Value');
% h = findobj(gcbf,'Tag','CheckboxSepa');
% if val == 1
%% Individuellement
% set(h,'Enable','off')
% else
%% En groupe
% set(h,'Enable','on')
% end

case 'ok'
% PushbuttonOk : prise en compte des parametres actuels, affichage
 h = findobj(gcbf,'Tag','ListboxDonneesFs');
  liste = get(h,'String');
  ud = get(gcbf,'UserData');
  repdon = ud{1};
  nomfifs = [repdon,liste{get(h,'Value')}];
 h = findobj(gcbf,'Tag','CheckboxNomfi');
  affnfi = isequal(get(h,'Value'), get(h,'Max'));
 h = findobj(gcbf,'Tag','PopupMenuOutput');
  typsor = get(h,'Value');
  liste = get(h,'UserData');
  driver = [liste{typsor}];
 h = findobj(gcbf,'Tag','CheckboxPara');
  param = isequal(get(h,'Value'), get(h,'Max'));
 h = findobj(gcbf,'Tag','PopupMenuTaicar');
  liste = get(h,'String');
  taicar = sscanf(liste{get(h,'Value')},'%d');
 h = findobj(gcbf,'Tag','PopupMenuMode');
  val = get(h,'Value');
 typf = [ud{2} 2];
 if val == 1
% Individuellement
  fnsing(nomfifs,affnfi,driver,param,typf,taicar)
 else
% En groupe
  fnsingg(nomfifs,affnfi,driver,param,typf,taicar)
 end

otherwise
 disp(['Error in uifsf, case not found : ',action])

end
