function uitracf(action)
%
%  Representation graphique des exposants de singularites.
%  Fonction regroupant les differentes actions a realiser selon les
%  choix effectues via l'interface graphique conversationnelle.
%

switch action

% ========================
% Definition des CreateFcn
% ========================

% Recuperation du repertoire racine, generation du repertoire des donnees
% creation d'une figure attachee a la figure maitresse.
% -----------------------------------------------------------------------
case 'iniappli'
% FigMaitresseRac
% !echo $HOME >fichierHOME
% fid = fopen('fichierHOME');
% home=fscanf(fid,'%s');
% !rm fichierHOME
 ficdir = 'exsiel.dir';
 fid = fopen(ficdir);
 if fid > 0
  repdon = fscanf(fid,'%s');
 else
  repdon = cd;
  disp(['*** File ', ficdir, ' not found.'])
  disp('Looking for data files in the current directory')
  disp(repdon)
 end
 set(gcbf,'UserData',[{[repdon,'/']},{gcbf}])
 uitracf creafig

% Initialisation des couleurs
% ---------------------------
case 'inicoul'
% PopupMenuCoulRe, PopupMenuCoulCpx
 liscoul = [{'yellow'},{'magenta'},{'cyan'},{'red'},{'green'}, ...
	    {'blue'},{'white'},{'black'}];
 indic = [{'y'},{'m'},{'c'},{'r'},{'g'},{'b'},{'w'},{'k'}];
 set(gcbo,'String',liscoul,'UserData',indic)

case 'inicoulcpx'
% PopupMenuCoulCpx
 uitracf inicoul;
 set(gcbo,'Value',6)

case 'inicoulre'
% PopupMenuCoulRe
 uitracf inicoul;
 set(gcbo,'Value',4)

% Initialisation des drivers de sortie
% ------------------------------------
case 'inioutput'
% PopupMenuOutput
 lisdri = [{'None'},{'Postscript'},{'Color Postscript'}];
 option = [{''},{'-deps2'},{'-depsc2'}];
 set(gcbo,'String',lisdri,'UserData',option)

% Initialisation des styles de representation
% -------------------------------------------
case 'inisty'
% PopupMenuStyRe, PopupMenuStyCpx
 lissty = [{'point'},{'circle'},{'x-mark'},{'plus'},{'star'}, ...
	   {'square'},{'diamond'},{'triangle (down)'}, ...
	   {'triangle (up)'},{'triangle (left)'}, ...
	   {'triangle (right)'},{'pentagram'},{'hexagram'}];
 indic = [{'.'},{'o'},{'x'},{'+'},{'*'},{'s'},{'d'},{'v'}, ...
	  {'^'},{'<'},{'>'},{'p'},{'h'}];
 set(gcbo,'String',lissty,'UserData',indic)

case 'inistycpx'
% PopupMenuStyCpx
 uitracf inisty;
 set(gcbo,'Value',2)

case 'inistyre'
% PopupMenuStyRe
 uitracf inisty;
 set(gcbo,'Value',1)

% Initialisation de l'affichage des tailles de caracteres
% -------------------------------------------------------
case 'initaicar'
% PopupMenuTaicar
 listai = [{'9'},{'10'},{'11'},{'12'},{'13'},{'14'},{'15'},{'16'}];
 set(gcbo,'String',listai,'Value',4)

% Initialisation de l'affichage des textes
% ----------------------------------------
case 'initraccpx'
% StaticTextRaccpx
 set(gcbo,'String',[{'Complex'},{'Exponent'}])

case 'initracre'
% StaticTextRacre
 set(gcbo,'String',[{'Real'},{'Exponent'}])


case 'iniudlim'
% PushbuttonLimit : Initialisation des donnees associees
 set(gcbo,'UserData',cell(1,2))

% Initialisation de l'affichage de la listbox contenant
% la liste des fichiers de donnees (d'extension .exp)
% -----------------------------------------------------
case 'lisfidon'
% ListboxDonneesRac
 repdon = get(gcbf,'UserData');
 repdon = repdon{1};
 strar = dir([repdon,'*.exp']);
 celar = struct2cell(strar);
%  On selectionne les fichiers, en ecartant les repertoires, puis
%  on trie par ordre alphabetique.
 ind = find([strar.isdir] == 0);
 set(gcbo,'String',cellstr(sortrows(char(celar(1,ind)))),'Max',length(ind))

% =======================
% Definition des Callback
% =======================
case 'choifidon'
% ListboxDonneesRac : reinitialisation des limites d'axes,
%		      de la partie reelle max
 h = findobj(gcbf,'Tag','EditTextRemax');
 set(h,'String','')
 h = findobj(gcbf,'Tag','PushbuttonLimit');
 set(h,'Enable','off','UserData',[{[]},{[]}])
 h = findobj(gcbf,'Tag','EditTextXmin');
 set(h,'String','','Enable','off')
 h = findobj(gcbf,'Tag','EditTextXmax');
 set(h,'String','','Enable','off')
 h = findobj(gcbf,'Tag','EditTextYminRe');
 set(h,'String','','Enable','off')
 h = findobj(gcbf,'Tag','EditTextYmaxRe');
 set(h,'String','','Enable','off')
 h = findobj(gcbf,'Tag','EditTextYmaxIm');
 set(h,'String','','Enable','off')

case 'fin'
% SubuimenuConfirm : fin de l'application, fermeture des fenetres
 ud = get(gcbf,'UserData');
 h = ud{2};
 if ishandle(h), close(h), end
 close(gcbf)

case 'help'
% uimenuHelp : aide sur cette application
 helptext=[
 ['                                                                               ']
 ['Data file   : Choose a data file by selecting an item. You can select several  ']
 ['              data files by pressing the SHIFT key as you click in the list.   ']
 ['              In this case, all the curves will be drawn on the same graph.    ']
 ['              This allows to compute separately several pieces of a curve and  ']
 ['              to concatenate them in one figure.                               ']
 ['Max Re(nu)  : If you give a value here, every exponent with real part greater  ']
 ['              than this value will be discarded when reading the data.         ']
 ['X Label     : Editable text to be printed under the graph.                     ']
 ['Axes limits : Updated with default plot values at each new data file selection.']
 ['              Once a graph is plotted, you can modify the values and then click']
 ['              on the "Update graph" button to update the graph immediately,    ']
 ['              without re-reading the data.                                     ']
 ['Title       : Switches on and off the display of a text at the top of the graph']
 ['              (by default, the name of the data file).                         ']
 ['              This text is managed by the function "gestit.m". You can write   ']
 ['              your own "gestit.m" file and put it (with the same name) in your ']
 ['              output directory to allow Matlab to find it. It will thus take   ']
 ['              presedence of the default function.                              ']
 ['              Type "help gestit" or "type gestit" for technical details.       ']
 ['Y Label     : Switches on and off the display of the ordinate legend on the    ']
 ['              left side of the graph.                                          ']
 ['Legend      : Switches on and off the display of the legend corresponding to   ']
 ['              the exponents representation, when there are complex ones.       ']
 ['Font size   : Sets the size of every text in the graph.                        ']
 ['Complex exp.: Sets the marker and color to be used to represent the complex    ']
 ['              exponents.                                                       ']
 ['Real exp.   : Sets the marker and color to be used to represent the real       ']
 ['              exponents.                                                       ']
 ['Output      : Sets the type of output file desired. Default is no output.      ']
 ['              The name of the file, created in your output directory, is given ']
 ['              in the Matlab window. When there is more than one data file, the ']
 ['              output filename is built upon the first of them.                 ']
 ['OK          : Click here to get a new graph taking into account all the        ']
 ['              previous choices.                                                ']
 ];
 disp(helptext)

case 'majlim'
% PushbuttonLimit : mise a jour des limites d'axes sur le graphique
 uitracf lirlim
 ud = get(gcbo,'UserData');
 limites = ud{2};
 if ~isempty(limites), modiax(ud{1},limites), end

case 'ok'
% PushbuttonOk : prise en compte des parametres actuels, affichage,
%                creation eventuelle d'un fichier postscript
 h = findobj(gcbf,'Tag','ListboxDonneesRac');
  liste = get(h,'String');
  repdon = get(gcbf,'UserData');
  repdon = repdon{1};
  lisfic = [];
  for indfic=get(h,'Value')
   lisfic = strvcat(lisfic,[repdon,liste{indfic}]);
  end
 h = findobj(gcbf,'Tag','CheckboxTitre');
  afftit = isequal(get(h,'Value'), get(h,'Max'));
 h = findobj(gcbf,'Tag','CheckboxLegY');
  labely = isequal(get(h,'Value'), get(h,'Max'));
 h = findobj(gcbf,'Tag','CheckboxLegExp');
  legrac = isequal(get(h,'Value'), get(h,'Max'));
 h = findobj(gcbf,'Tag','PopupMenuTaicar');
  liste = get(h,'String');
  taicar = sscanf(liste{get(h,'Value')},'%d');
 h = findobj(gcbf,'Tag','EditTextLegX');
  legendx = get(h,'String');
 h = findobj(gcbf,'Tag','EditTextRemax');
  chaine = get(h,'String');
  if isempty(chaine), prmax = realmax;
  else, prmax = sscanf(chaine,'%f'); end
 h = findobj(gcbf,'Tag','PopupMenuCoulCpx');
  liste = get(h,'UserData');
  styc = [liste{get(h,'Value')}];
 h = findobj(gcbf,'Tag','PopupMenuStyCpx');
  liste = get(h,'UserData');
  styc = [styc,liste{get(h,'Value')}];
 h = findobj(gcbf,'Tag','PopupMenuCoulRe');
  liste = get(h,'UserData');
  styr = [liste{get(h,'Value')}];
 h = findobj(gcbf,'Tag','PopupMenuStyRe');
  liste = get(h,'UserData');
  styr = [styr,liste{get(h,'Value')}];

 uitracf lirlim
 h = findobj(gcbf,'Tag','PushbuttonLimit');
 ud = get(h,'UserData');
 limites = ud{2};

% On rend courante la figure associee. Elle est protegee (cf sa propriete
% HandleVisibility). Cependant, une figure n peut toujours etre detruite
% a partir de la ligne de commande par la commande "close(n)". C'est
% pourquoi on teste si elle existe toujours ; on la recree si besoin.
 ud = get(gcbf,'UserData');
 h = ud{2};
 if ishandle(h), figure(h), else uitracf creafig, end
 clf
% On repere ici son numero pour l'utiliser comme parametre a la fonction
% d'impression et non plus se baser sur la notion de fenetre courante qui
% varie au cours des instructions suivantes.
 curfig = gcf;

% Traitement du fichier de donnees avec les parametres courants
 [haxes, hplot] = exploit(lisfic,prmax,styr,styc,taicar, ...
                          legendx,labely,afftit,legrac,limites);

 if isempty(haxes) | isempty(hplot), return, end

% Mise a jour des valeurs courantes des limites d'axes
 xlim = get(haxes(1),'Xlim');
 xmin = num2str(xlim(1)); xmax = num2str(xlim(2));
 h = findobj(gcbf,'Tag','EditTextXmin');
 set(h,'String',xmin,'Enable','on')
 h = findobj(gcbf,'Tag','EditTextXmax');
 set(h,'String',xmax,'Enable','on')
 ylim = get(haxes(1),'Ylim');
 yminr = num2str(ylim(1)); ymaxr = num2str(ylim(2));
 h = findobj(gcbf,'Tag','EditTextYminRe');
 set(h,'String',yminr,'Enable','on')
 h = findobj(gcbf,'Tag','EditTextYmaxRe');
 set(h,'String',ymaxr,'Enable','on')
 if length(haxes) > 1
  ylim = get(haxes(2),'Ylim'); ymaxi = num2str(ylim(2));
  h = findobj(gcbf,'Tag','EditTextYmaxIm');
  set(h,'String',ymaxi,'Enable','on')
 else
  ymaxi = [];
 end
 h = findobj(gcbf,'Tag','PushbuttonLimit');
 set(h,'Enable','on','UserData',[{haxes},{[xmin xmax yminr ymaxr ymaxi]}])

% Creation eventuelle d'un fichier Postscript correspondant au dessin actuel
 h = findobj(gcbf,'Tag','PopupMenuOutput');
 typsor = get(h,'Value');
 if typsor > 1
  liste = get(h,'UserData');
  driver = [liste{typsor}];
  h = findobj(gcbf,'Tag','ListboxDonneesRac');
  liste = get(h,'String');
  indfic=get(h,'Value');
  nomfic = liste{indfic(1)};
  dernc = min([length(nomfic)+1 find(nomfic=='.')]) - 1;
  nomfic = [nomfic(1:dernc),'.ps'];
  print(curfig, driver, [repdon,nomfic])
  disp(['Created file : ',nomfic,'      (Print option : ',driver,')'])
 end

% ===============================
% Definition des actions internes
% ===============================
case 'creafig'
% Creation d'une figure destinee a recevoir le graphique
 ud = get(gcbf,'UserData');
 ud{2}=figure;
 set(gcbf,'UserData',ud)
 p1 = get(gcbf,'Position');
 p2 = get(ud{2},'Position');
 p2(1) = p1(1);
 p2(2) = p1(2)-p2(4);
 set(ud{2},'Position',p2)
 set(ud{2},'HandleVisibility','callback','MenuBar','none')
 set(ud{2},'Name','Singularity exponents','NumberTitle','off')

case 'lirlim'
% Lecture des limites d'axes, memorisation dans la zone UserData
% du PushbuttonLimit
 hl = findobj(gcbf,'Tag','PushbuttonLimit');
 ud = get(hl,'UserData');
 if ~isempty(ud{1})
  h = findobj(gcbf,'Tag','EditTextXmin');
   [xmin, ncarlus] = sscanf(get(h,'String'),'%f');
% Sans le test suivant, xmin serait de type char avec Matlab 6.
% Idem pour les suivants.
   if ncarlus == 0, xmin = []; end
  h = findobj(gcbf,'Tag','EditTextXmax');
   [xmax, ncarlus] = sscanf(get(h,'String'),'%f');
   if ncarlus == 0, xmax = []; end
  h = findobj(gcbf,'Tag','EditTextYminRe');
   [yminr, ncarlus] = sscanf(get(h,'String'),'%f');
   if ncarlus == 0, yminr = []; end
  h = findobj(gcbf,'Tag','EditTextYmaxRe');
   [ymaxr, ncarlus] = sscanf(get(h,'String'),'%f');
   if ncarlus == 0, ymaxr = []; end
  h = findobj(gcbf,'Tag','EditTextYmaxIm');
   [ymaxi, ncarlus] = sscanf(get(h,'String'),'%f');
   if ncarlus == 0, ymaxi = []; end
  ud{2} = [xmin xmax yminr ymaxr ymaxi];
 end
 set(hl,'UserData',ud);

otherwise
 disp(['Error in uitracf, case not found : ',action])

end
