************************************************************************
*  Calcul d'exposants de singularites en elasticite anisotrope.
*  Yvon Lafranche, Universite de Rennes 1.
*  Voir documentation a l'adresse :
*        http://perso.univ-rennes1.fr/yvon.lafranche/exsiel/
************************************************************************
************************************************************************
* ANALYS : analyse de la multiplicite sur le rectangle de depart
* CALMUL : calcul de la multiplicite dans les rectangles separateurs
* DMULTI : determination de la multiplicite
* DNBRAC : determination du nombre de racines dans le rectangle initial
* ESTIPR : estimation precise d'une racine
* EVAFNA : evaluation de la fonction d'analyse de la multiplicite
* IMPRAC : impression des racines dans un fichier
* INCAUC : calcul d'integrales de Cauchy sur un cercle
* INCAUR : calcul d'integrales de Cauchy sur un rectangle
* NSSCAD : nouveau sous-cadre de la subdivision
* PILDEP : depilement d'un cadre
* PILEMP : empilement d'un cadre
* QUADRI : quadrillage d'un rectangle pour la recherche de racines
* RACLAS : classement des racines
* RACDEP : depilement d'une racine dans la pile
* RACEMP : empilement d'une racine dans la pile
* RAYON  : distance d'un point au bord d'un rectangle
* SCIND  : tentative pour "scinder" en 2 une racine double
* SEPRAC : separation des racines
* SOMPAR : calcul de sommes partielles pour l'integrale de Cauchy
* SUBCAD : subdivision d'un cadre
* SUICAL : suivi du calcul
* TRCAD  : traitement d'un cadre 
* TRCAM2 : traitement du cas ou la multiplicite vaut 2
* TSTCAD : test de la subdivision d'un cadre
* TSTCOT : test d'un cote d'un rectangle
************************************************************************
      subroutine ANALYS(nbprec,som,ninter,nbabs,absci,codret)
      integer nbprec, ninter, nbabs, codret
      real som(4), absci(ninter)
*  Variables locales
      integer i, ideb, ifin, ecart
      real h, fnap, marge, somp(4)
      logical const
*       ------------------------------------------------------------
*       Analyse de la multiplicite lorsque le rectangle de recherche
*       des racines est deforme dans le sens des x croissants.
*       Ce sous-programme renvoie une liste d'abscisses qui separent
*       les racines.
*       Arguments d'entree  : nbprec,som,ninter
*       Arguments de sortie : nbabs,absci
*       ------------------------------------------------------------
      codret = 0
      h = (som(2)-som(1)) / ninter
      marge = (som(2)-som(1)) / 10.
      do 10 i=1,4
   10  somp(i) = som(i)
      ecart = 1 + int(ninter/30.)
*
*       Le bord du rectangle de recherche est un separateur.
*       (necessaire si la 1ere zone constante n'est pas assez large)
      nbabs = 1
      absci(nbabs) = som(1)
*
*       Calcul de la fonction d'analyse et determination
*       des intervalles ou elle est "a peu pres" constante.
      fnap = -1.
      i = 0
   20 i = i + 1
       if (i .gt. ninter) goto 40
       somp(2) = som(1) + i*h
       call EVAFNA(nbprec,somp,fnap,const)
      if (.not. const) goto 20
*       On a trouve le debut d'une zone constante
*       (on repere toujours le debut d'un intervalle)
      ideb = i-1
*
*       On en recherche la fin. Ici, il est inutile de tester i par
*       rapport a ninter, la fin de la zone peut etre au dela de som(2).
   30 i = i + 1
       somp(2) = som(1) + i*h
       call EVAFNA(nbprec,somp,fnap,const)
      if (const) goto 30
      ifin = i-1
*
      if (ifin-ideb .gt. ecart) then
*       La zone est assez large pour determiner une frontiere
       nbabs = nbabs + 1
       if (nbabs .gt. ninter) then
        codret = 1
        return
       endif
       absci(nbabs) = som(1) + (ideb+ifin)*h*0.5
      endif
      if (som(2)-absci(nbabs) .gt. marge) goto 20
      return
*
   40 nbabs = nbabs + 1
      if (nbabs .gt. ninter) then
       codret = 1
       return
      endif
      absci(nbabs) = som(2)
*
CC
CC Aussi dans EVAFNA
CC      write(55,*) 0, 0, 99, 0, 0
CC
      end
************************************************************************
      subroutine CALMUL(nbprec,ks,nivsub,izkdpd,nbabs,absci,sommod,
     & sompil,codret)
      integer nbprec,ks,nivsub,nbabs,sompil,codret
      real absci(nbabs), sommod(4)
      complex izkdpd(0:ks)
*  Variables locales
      integer i, multi
      real som(4)
      logical premier
*       -------------------------------------------------------
*       Calcul de la multiplicite dans les rectangles
*       separateurs de racines de la forme
*          [absci(i), absci(i+1)] x [sommod(3),sommod(4)].
*       Ce sous-programme renvoie dans sommod(1) et sommod(2)
*       les abscisses des separateurs extremes reellement
*       utilises (car il se peut que certains rectangles de la
*       forme precedente soient vides de racines).
*       Arguments d'entree  : nbprec,ks,nivsub,nbabs,absci
*                             sommod,sompil
*       Arguments de sortie : sommod,sompil,codret
*       Tableaux de travail : izkdpd
*        codret : code de retour egal a 0 si RAS,
*                 1 si la pile est pleine, 2 si la multiplicite
*                 est incorrecte dans un rectangle
*       -------------------------------------------------------
      som(3) = sommod(3)
      som(4) = sommod(4)
      premier = .true.
*
      som(1) = absci(1)
      do 10 i=2,nbabs
       som(2) = absci(i)
       call INCAUR(som,nbprec,ks,izkdpd)
       call DMULTI(izkdpd(0),nivsub,multi)
       if (multi .gt. 0) then
        call PILEMP(som,nivsub,multi,izkdpd(1),sompil,codret)
        if (premier) then
         premier = .false.
         sommod(1) = som(1)
        endif
        sommod(2) = som(2)
       elseif (multi .lt. 0) then
        codret = 2
        return
       endif
CC
CC       write(56,*) som(2), multi
CC
       som(1) = som(2)
   10 continue
CC
CC      write(56,*)
CC
      end
************************************************************************
      subroutine DMULTI(z,nivsub,m)
      complex z
      integer nivsub,m
*  Variables locales
      integer mul, nbnivx, multix
      parameter (nbnivx = 6, multix = 50)
      real eps, r, tabeps(nbnivx)
      data tabeps /4.e-2, 2.e-2, 1.e-2, 8.e-3, 6.e-3, 4.e-3/
*       ----------------------------------------------------------
*       Determination de la multiplicite, egale normalement a un
*       entier >= 0, dont la partie relle de z est censee etre une
*       bonne approximation. Le calcul de l'integrale de Cauchy de
*       f'/f ne conduit pas necessairement a une valeur coherente
*       (proche d'un entier positif et inferieure a une limite
*       superieure fixee a multix) si le contour passe a proximite
*       d'une racine. Dans ce cas, la routine renvoie m = -1 ; dans
*       le cas contraire, m prend la valeur de la multiplicite.
*       Arguments d'entree  : z,nivsub
*       Arguments de sortie : m
*        z      : valeur de l'integrale \int f'/f
*        nivsub : niveau courant de subdivision
*        m      : multiplicite cherchee
*       ----------------------------------------------------------
      eps = tabeps(min(nivsub,nbnivx))
      m = -1
      if (abs(aimag(z)) .lt. eps) then
       r = real(z)
       mul = nint(r)
       if (abs(r-mul).lt.eps .and. mul.ge.0 .and. mul.lt.multix) m = mul
      endif
      end
************************************************************************
      subroutine DNBRAC(prmin,nbprec,ks,som,nivsub,multi,izkdpd,codret)
      integer nbprec,ks,nivsub,multi,codret
      real prmin, som(4)
      complex izkdpd(0:ks)
*  Variables locales
      integer j, igd, ipt, rap, ngdpas, sens
      logical ok
      real pas, paslim, sp(4), limite, fract, s2, delta
      parameter (paslim = 0.05, fract = 20., rap = 10, ngdpas = 10)
*  Point d'entree
      real RAYON
*       ------------------------------------------------------------
*       Determination du nombre de racines situees dans le rectangle
*       de sommets som (xmin,xmax,ymin,ymax).
*       Le rectangle est eventuellement modifie de maniere a obtenir
*       une valeur fiable de ce nombre.
*       Arguments d'entree  : prmin,nbprec,ks,som,nivsub
*       Arguments de sortie : som,multi,izkdpd,codret
*        prmin  : valeur min de l'abscisse definissant le rectangle
*        nbprec : nombre de points pour le calcul des
*                 integrales sur un rectangle
*        ks     : nombre d'integrales a calculer
*        som    : en entree : sommets du cadre a tester
*                             (xmin, xmax, ymin, ymax)
*                 en sortie : cadre eventuellement deforme
*        nivsub : niveau courant de subdivision
*        multi  : multiplicite des racines contenues dans le cadre
*        izkdpd : integrales de contour sur le bord du rectangle
*        codret : code de retour egal a 0 si RAS, 1 en cas d'echec
*       ------------------------------------------------------------
      codret = 0
      call INCAUR(som,nbprec,ks,izkdpd)
      call DMULTI(izkdpd(0),nivsub,multi)
*      Controle de coherence : le centre de gravite des racines
*      doit se situer a l'interieur du rectangle selectionne.
      if (multi .eq. 0) goto 40
      if (multi .gt. 0) then
       if (RAYON(izkdpd(1)/multi,som) .gt. 0.) goto 40
      endif
*       La multiplicite n'est pas acceptable : une racine est
*       proche du bord du rectangle.
*       On teste d'abord les cotes 1, 3 et 4 en recherchant eventuellement
*       une zone sans racine a proximite et a l'exterieur du rectangle,
*       ceci car c'est le cote 2 qui statistiquement pose le plus de
*       problemes.
*       On teste enfin globalement le rectangle issu de cette recherche,
*       en decalant eventuellement le cote 2 d'abord de part et d'autre
*       de sa position d'origine, puis en cas d'echec, dans le sens
*       positif, jusqu'a atteindre un rectangle de longueur double
*       auquel cas on conclue a un echec.
*
*       Cote 1 (xmin)
      limite = max(prmin,som(1)-(som(2)-som(1))/2.)
      pas = min(paslim,2.*(som(1)-limite)/fract)
      sp(1) = som(1) - pas
      sp(2) = som(1)
      sp(3) = som(3)
      sp(4) = som(4)
      call TSTCOT(nbprec,1,-pas,limite,nivsub,sp,ok)
      if (.not. ok) goto 30
      som(1) = sp(2)
*       Cote 3 (ymin)
      pas = min(paslim,(som(4)-som(3))/fract)
      sp(1) = som(1)
      sp(2) = som(2)
      sp(3) = som(3) - pas
      sp(4) = som(3)
      limite = som(3)-(som(4)-som(3))/2.
      call TSTCOT(nbprec,3,-pas,limite,nivsub,sp,ok)
      if (.not. ok) goto 30
      som(3) = sp(4)
*       Cote 4 (ymax)
      sp(3) = som(4)
      sp(4) = som(4) + pas
      limite = som(4)+(som(4)-som(3))/2.
      call TSTCOT(nbprec,3, pas,limite,nivsub,sp,ok)
      if (.not. ok) goto 30
      som(4) = sp(3)
*
*       Cote 2 (xmax) : test global avec calcul de la multiplicite.
      s2 = som(2)
      pas = (som(2)-som(1)) / (2*rap*ngdpas)
*       On teste d'abord de part et d'autre de la frontiere
*       sur une largeur de 2 bandes.
      do 10 igd=0,1
       j = igd*rap
       do 10 ipt=rap,1,-1
        delta = (ipt+j)*pas
        do 10 sens=1,-1,-2
         som(2) = s2 + sens*delta
         call INCAUR(som,nbprec,ks,izkdpd)
         call DMULTI(izkdpd(0),nivsub,multi)
CC       WRITE(*,'(a,4f6.3,2e21.14,i4)') 'DNBRAC', som, izkdpd(0),multi
CC       WRITE(*,'(2e21.14)') izkdpd(1)
         if (multi .eq. 0) goto 40
         if (multi .gt. 0) then
          if (RAYON(izkdpd(1)/multi,som) .gt. 0.) goto 40
         endif
   10 continue
*       On teste maintenant uniquement dans le sens d'un
*       accroissement du domaine.
      do 20 igd=2,ngdpas-1
       j = igd*rap
       do 20 ipt=rap,1,-1
        som(2) = s2 + (ipt+j)*pas
        call INCAUR(som,nbprec,ks,izkdpd)
        call DMULTI(izkdpd(0),nivsub,multi)
CC       WRITE(*,'(a,4f6.3,2e21.14,i4)') 'DNBRAC', som, izkdpd(0),multi
CC       WRITE(*,'(2e21.14)') izkdpd(1)
        if (multi .eq. 0) goto 40
        if (multi .gt. 0) then
         if (RAYON(izkdpd(1)/multi,som) .gt. 0.) goto 40
        endif
   20 continue
*
   30 codret = 1
   40 return
      end
************************************************************************
      subroutine ESTIPR(nbpcer,toler,som,nivsub,ks,multi,z,r,izkdpd,
     & nsubut,codret)
      integer nbpcer,nivsub,ks,multi,nsubut,codret
      real toler, som(4), r
      complex z, izkdpd(0:ks)
*  Variables locales
      integer mul
      logical test
      complex centre, cdg
*  Point d'entree
      real RAYON
*       -------------------------------------------------------------
*       Calcul d'une estimation plus precise de la racine z situee
*       dans le rectangle de sommets som. On calcule des integrales
*       de contour sur des cercles de rayon de plus en plus petits.
*       z(n) et z(n+1) etant deux approximations successives, on
*       arrete les calculs :
*         . si | z(n) - z(n+1) | < toler (succes)
*         . ou si la multiplicite devient differente de multi (echec)
*         . ou si le centre de gravite des racines est situe a
*           l'exterieur du cercle (echec)
*       Arguments d'entree  : nbpcer,toler,som,nivsub,ks,multi,z
*       Arguments de sortie : z,r,izkdpd,nsubut,codret
*        nbpcer : nombre de points pour le calcul des
*                 integrales sur un cercle
*        toler  : precision requise pour le calcul des exposants
*        som    : sommets du cadre contenant la racine
*        nivsub : niveau courant de subdivision
*        ks     : dimension de izkdpd
*        multi  : multiplicite de la racine
*        z      : en entree, valeur approchee de la racine * multi
*                 en sortie, valeur precise de la racine * multi en
*                 cas de succes, derniere approximation calculee sinon
*        r      : dernier rayon utilise pour le calcul des integrales
*        izkdpd : integrales de Cauchy calculees sur le cercle de
*                 rayon r jusqu'a l'ordre ks
*        nsubut : dernier niveau de subdivision utilise
*        codret : code de retour egal a 0 si RAS, 1 en cas d'echec
*       -------------------------------------------------------------
      centre = z/multi
      r = RAYON(centre,som)
      if (r .lt. 0.) goto 20
      codret = 0
*
      nsubut = nivsub + 1
      call INCAUC(centre,r,nbpcer,ks,izkdpd)
      call DMULTI(izkdpd(0),nsubut,mul)
      if (mul .ne. multi) goto 20
      cdg = izkdpd(1)/multi
      if (abs(cdg-centre) .gt. r) goto 20
      z = izkdpd(1)
*
   10 r = r*0.5
      nsubut = nsubut + 1
      centre = cdg
      call INCAUC(centre,r,nbpcer,ks,izkdpd)
      call DMULTI(izkdpd(0),nsubut,mul)
      if (mul .ne. multi) goto 20
      cdg = izkdpd(1)/multi
      if (abs(cdg-centre) .gt. r) goto 20
      test = abs(z-izkdpd(1)) .gt. toler
      z = izkdpd(1)
      if (test) goto 10
      return
*
*      Echec
   20 codret = 1
      end
************************************************************************
      subroutine EVAFNA(nbprec,somp,fnap,const)
      integer nbprec
      real somp(4), fnap
      logical const
*  Variables locales
      integer multi, multix
      real deltay, err, fnac
      complex izkdpd
*  Initialisations
      parameter (multix = 50, deltay = 0.1)
*       ------------------------------------------------------------
*       Evaluation de la fonction d'analyse de la multiplicite pour
*       le rectangle de recherche somp.
*       Arguments d'entree  : nbprec,somp,fnap
*       Arguments de sortie : fnap,const
*       ------------------------------------------------------------
      call INCAUR(somp,nbprec,0,izkdpd)
      if (abs(real(izkdpd)) .gt. multix) then
       multi = multix
       err = 1.
      else
       multi = abs(nint(real(izkdpd)))
       err = abs(izkdpd-multi)
      endif
CC
CC Aussi dans ANALYS
CC      write(55,'(2(1x,e17.10),1x,i3,1x,e12.5,1x,e17.10)')
CC     &real(izkdpd), aimag(izkdpd), multi, somp(2), err+multi
CC
      fnac = err+multi
      const = abs(fnac-fnap) .lt. deltay
      fnap = fnac
      end
************************************************************************
      subroutine IMPRAC(iurac,numrac,codret)
      integer iurac,numrac,codret
*  Variables locales
      integer i, multi
      complex z
*       -----------------------------------------------------------
*       Impression des racines calculees dans le fichier associe
*       a l'unite logique iurac. Les racines sont d'abord classees
*       par ordre croissant de partie reelle.
*       Arguments d'entree  : iurac,numrac
*       Arguments de sortie : codret
*        iurac  : unite logique du fichier des racines
*        numrac : indice indiquant le sommet de la pile de stockage
*                 des racines calculees
*        codret : code de retour egal a 0 si RAS, 2 en cas d'erreur
*                 d'ecriture dans le fichier.
*       -----------------------------------------------------------
      codret = 0
*      Classement des racines par ordre croissant de partie reelle
      call RACLAS(numrac)
*
*      Impression des racines avec leur multiplicite
      write(iurac,*,err=40) numrac
      do 10 i=1,numrac
       call RACDEP(i,multi,z)
   10  write(iurac,50,err=40) z, multi
      return
*
   40 codret = 2
*
   50 format(2(1x,g23.15e3),1x,i2)
      end
************************************************************************
      subroutine INCAUC(z0,r,n,ks,izkdpd)
      integer n, ks
      complex z0, izkdpd(0:ks)
      real r
*  Variables locales
      integer j, k
      real h, tj
      complex z, eitj, eitjf
*  Point d'entree
      complex DPZSDZ
*       --------------------------------------------------------------
*       Calcul des integrales de Cauchy :
*        izkdpd(k) = 1/(2 i pi) \int z**k d'(z)/d(z) dz, k=0,ks
*       ou d est le determinant caracteristique.
*       Il s'agit d'integrales de contour sur le cercle de centre z0
*       et de rayon r. L'integration se fait par la methode des
*       trapezes, avec n points sur le contour repartis uniformement.
*       Arguments d'entree  : z0,r,n,ks
*       Arguments de sortie : izkdpd
*        z0, r : centre et rayon du cercle contour d'integration
*        n   : nombre de points de calcul de la methode des trapezes
*        ks  : exposant de z dans la derniere integrale a calculer
*        izkdpd(k) : integrales recherchees (k=0,ks) representant
*          respectivement la multiplicite (k=0), la somme des racines
*          de l'equation d(z) = 0 (k=1), la somme de leurs carres
*          (k=2), etc...
*       --------------------------------------------------------------
*      Pas = 2 pi / n
      h = 8.*atan(1.) / real(n)
*
      do 10 k=0,ks
   10  izkdpd(k) = cmplx(0.)
      do 30 j=1,n
       tj = real(j)*h
       eitj = cmplx(cos(tj),sin(tj))
       z = z0 + r*eitj
       eitjf = eitj * DPZSDZ(z)
       izkdpd(0) = izkdpd(0) + eitjf
       do 20 k=1,ks
       eitjf = z*eitjf
   20  izkdpd(k) = izkdpd(k) + eitjf
   30 continue   
*
      h = r / real(n)
      do 40 k=0,ks
   40  izkdpd(k) = h * izkdpd(k)
      end
************************************************************************
      subroutine INCAUR(som,nbp,ks,izkdpd)
      real som(4)
      integer nbp, ks
      complex izkdpd(0:ks)
*  Variables locales
      integer nx,ny, k,ksl
      parameter (ksl = 2)
      real hx, hy, vdx,vdy
      complex z1,z2, fz1,fz2, sxz1(0:ksl),syz1(0:ksl)
      complex sxz2(0:ksl),syz2(0:ksl), sxz3(0:ksl),syz3(0:ksl)
*  Point d'entree
      complex DPZSDZ
*       ---------------------------------------------------------------
*       Calcul des integrales de Cauchy :
*        izkdpd(k) = 1/(2 i pi) \int z**k d'(z)/d(z) dz, k = 0,...ks
*       ou d est le determinant caracteristique.
*       Il s'agit d'integrales de contour sur le rectangle defini par
*       les points du plan complexe : a+ic, b+ic, b+id, a+id, donnes
*       dans le sens trigonometrique, avec (a,b,c,d) = (som(i),i=1,4).
*       L'integration se fait par la methode de Simpson, avec environ
*       4*nbp points sur le contour repartis uniformement en fonction
*       de la geometrie du rectangle (il y a nbp points par cote dans
*       le cas d'un carre).
*       Arguments d'entree  : som,nbp,ks
*       Arguments de sortie : izkdpd
*        som : sommets du rectangle contour d'integration
*        nbp : nombre de points de calcul de la methode de Simpson
*        ks  : exposant de z dans la derniere integrale a calculer
*       ---> ATTENTION : ks doit etre <= ksl, mais ceci n'est pas
*                        controle par ce sous-programme.
*        izkdpd(k) : integrales recherchees (k=0,ks) representant
*          respectivement la multiplicite (k=0), la somme des racines
*          de l'equation d(z) = 0 (k=1), la somme de leurs carres (k=2)
*       ---------------------------------------------------------------
*       Repartition des points d'integration sur le contour
      hx = som(2)-som(1)
      hy = som(4)-som(3)
      if (hx .gt. hy) then
       nx = nbp
       ny = max(1, int(hy/hx * nbp + 0.1))
      else
       nx = max(1, int(hx/hy * nbp + 0.1))
       ny = nbp
      endif
      vdx = 2.*nbp / real(nx+ny)
      nx = max(1, int(nx*vdx+0.5))
      ny = max(1, int(ny*vdx+0.5))
      hx = hx / nx
      hy = hy / ny
*
      z1 = cmplx(som(1),som(3))
      z2 = cmplx(som(1),som(4))
      fz1 = DPZSDZ(z1)
      fz2 = DPZSDZ(z2)
      sxz1(0) =  fz1 - fz2
      syz1(0) = -fz1 - fz2
      do 10 k=1,ks
       fz1 = z1*fz1
       fz2 = z2*fz2
       sxz1(k) =  fz1 - fz2
   10  syz1(k) = -fz1 - fz2
*
      z1 = cmplx(som(2),som(3))
      z2 = cmplx(som(2),som(4))
      fz1 = DPZSDZ(z1)
      fz2 = DPZSDZ(z2)
      sxz1(0) = sxz1(0) + fz1 - fz2
      syz1(0) = syz1(0) + fz1 + fz2
      do 20 k=1,ks
       fz1 = z1*fz1
       fz2 = z2*fz2
       sxz1(k) = sxz1(k) + fz1 - fz2
   20  syz1(k) = syz1(k) + fz1 + fz2
*
      vdx = som(1)
      vdy = som(3)
      call SOMPAR(1,nx-1,ny-1,hx,hy,vdx,vdy,som,ks,sxz2,syz2)
*
      vdx = som(1) + hx*0.5
      vdy = som(3) + hy*0.5
      call SOMPAR(0,nx-1,ny-1,hx,hy,vdx,vdy,som,ks,sxz3,syz3)
*
*        z1 = 1/(12 i pi), z2 = i
      z1 = cmplx(0.,48.*atan(1.))
      z2 = cmplx(0.,1.)
      do 30 k=0,ks
       izkdpd(k) = (hx*(sxz1(k)+2*sxz2(k)+4*sxz3(k)) + 
     &           z2*hy*(syz1(k)+2*syz2(k)+4*syz3(k))) / z1
   30 continue
      end
************************************************************************
      subroutine NSSCAD(sx,sy,numsc,typsub,sp)
      real sx(3),sy(3), sp(4)
      integer numsc, typsub
*  Variables locales
      integer ix, iy
*       -------------------------------------------------------
*       Determination du nouveau sous-cadre de la subdivision
*       du rectangle dans lequel on recherche les racines.
*       Arguments d'entree  : sx,sy,numsc,typsub
*       Arguments de sortie : sp
*        sx, sy : tableau contenant les abscisses et ordonnees
*                 definissant le decoupage
*        numsc  : numero du sous-cadre concerne
*        typsub : type de la subdivision (1 = en 2 selon x,
*                 2 = en 2 selon y, 3 = en 4, 4 = en 3 selon y)
*        sp     : sommets du sous-cadre a determiner
*       -------------------------------------------------------
      if (typsub .eq. 1) then
*       Subdivision en 2 selon x
       sp(1) = sx(numsc)
       sp(2) = sx(numsc+1)
       sp(3) = sy(1)
       sp(4) = sy(3)
      elseif (typsub .eq. 2) then
*       Subdivision en 2 selon y
       sp(1) = sx(1)
       sp(2) = sx(3)
       sp(3) = sy(numsc)
       sp(4) = sy(numsc+1)
      elseif (typsub .eq. 3) then
*       Subdivision en 4
       ix = mod(numsc-1,2) + 1
       iy = (numsc-1)/2 + 1
       sp(1) = sx(ix)
       sp(2) = sx(ix+1)
       sp(3) = sy(iy)
       sp(4) = sy(iy+1)
      else
*       Subdivision en 3 selon y
       sp(1) = sx(1)
       sp(2) = sx(3)
       if (numsc .eq. 1) then
        sp(3) = sy(1) + sy(3)-sy(2)
        sp(4) = sy(2)
       elseif (numsc .eq. 2) then
        sp(3) = sy(2)
        sp(4) = sy(3)
       else
        sp(3) = sy(1)
        sp(4) = sy(1) + sy(3)-sy(2)
       endif
      endif
*
      end
************************************************************************
      subroutine PILDEP(som,nivsub,mul,int,sompil)
      real som(4)
      integer nivsub, mul, sompil
      complex int(2)
*  Variables locales
      integer i
*  Communs
      integer tpilx
      parameter (tpilx = 100)
      integer pmul(tpilx), pniv(tpilx)
      real psom(4,tpilx)
      complex pintc(2,tpilx)
      common /pilcad/ pintc, psom, pniv, pmul
*       ----------------------------------------------------------
*       Depilement du cadre de sommets (som(i),i=1,4)
*       et des valeurs associees.
*       En sortie, sompil = 0 indique que la pile est vide.
*       Arguments d'entree  : sompil
*       Arguments de sortie : som,nivsub,mul,int,sompil
*        som : sommets du cadre a depiler
*        nivsub,mul,int : valeurs associees a depiler
*        sompil : indice indiquant le sommet de la pile des cadres
*       ----------------------------------------------------------
      do 10 i=1,4
   10  som(i) = psom(i,sompil)
      nivsub = pniv(sompil)
      mul    = pmul(sompil)
      int(1) = pintc(1,sompil)
      int(2) = pintc(2,sompil)
      sompil = sompil - 1
      end
************************************************************************
      subroutine PILEMP(som,nivsub,mul,int,sompil,codret)
      real som(4)
      integer nivsub, mul, sompil, codret
      complex int(2)
*  Variables locales
      integer i
*  Communs
      integer tpilx
      parameter (tpilx = 100)
      integer pmul(tpilx), pniv(tpilx)
      real psom(4,tpilx)
      complex pintc(2,tpilx)
      common /pilcad/ pintc, psom, pniv, pmul
*       ----------------------------------------------------------
*       Empilement du cadre de sommets (som(i),i=1,4)
*       et des valeurs associees.
*       Arguments d'entree  : som,nivsub,mul,int,sompil
*       Arguments de sortie : sompil,codret
*        som : sommets du cadre a empiler
*        nivsub,mul,int : valeurs associees a empiler
*        sompil : indice indiquant le sommet de la pile des cadres
*        codret : code de retour egal a 0 si RAS,
*                 1 si la pile est pleine.
*       ----------------------------------------------------------
      sompil = sompil + 1
      if (sompil .le. tpilx) then
       codret = 0
       do 10 i=1,4
   10   psom(i,sompil) = som(i)
       pniv(sompil) = nivsub
       pmul(sompil) = mul
       pintc(1,sompil) = int(1)
       pintc(2,sompil) = int(2)
      else
       codret = 1
      endif
      end
************************************************************************
      subroutine QUADRI(ius,prmin,somrec,nbpcer,nbprec,pimin,taimai,
     & toler,nbsep,nbintx,absci,numrac,sommod)
      integer ius,nbpcer,nbprec,nbsep,nbintx,numrac
      real prmin, somrec(4),pimin,taimai,toler,absci(nbintx),sommod(4)
*  Variables locales
      integer ks
      parameter (ks = 2)
      integer i, nivsub, multi, sompil, codret, ninter, nbmes
      real som(4)
      complex izkdpd(0:ks)
      character*52 txtsup
*       ----------------------------------------------------------------
*       Quadrillage du rectangle (a,b)x(c,d) du plan complexe pour
*       la recherche de racines (exposants de singularite), avec
*       (a,b,c,d) = (somrec(i),i=1,4).
*       Les racines trouvees sont stockees dans une pile interne.
*       Arguments d'entree  : ius,prmin,somrec,nbpcer,nbprec,pimin
*                             taimai,toler,nbsep,nbintx,absci
*       Arguments de sortie : nbsep,numrac,sommod
*       Tableaux de travail : absci
*        ius    : numero de l'unite logique du fichier resultat
*        prmin  : valeur min de l'abscisse definissant le rectangle
*                 pour la recherche automatique de racines
*        somrec(1),somrec(2) : abscisses min et max du rectangle
*        somrec(3),somrec(4) : ordonnees min et max du rectangle
*        nbpcer,nbprec : nombre de points pour le calcul des
*                      integrales sur un cercle et sur un rectangle
*        pimin  : partie imaginaire min en deca de laquelle les
*                 exposants ne sont pas recherches
*        taimai : taille min de la maille rectangulaire lors du
*                 decoupage du plan pour la recherche des racines
*        toler  : precision requise pour le calcul des exposants
*        nbsep  : en entree : nombre de separateurs des zones ou sont
*                 localisees les racines, determine lors de l'appel
*                 precedent. N'est pas pris en compte s'il vaut 0.
*                 en sortie : remis a 0 en cas d'erreur dans ce module ;
*                 sinon, nombre de separateurs des zones de localisation
*                 des racines venant d'etre determinees.
*        nbintx : nombre max de separateurs de zones permis
*        absci  : abscisses des separateurs de zones contenant des
*                 racines, determinees lors de la phase d'analyse
*        numrac : indice indiquant le sommet de la pile de stockage
*                 des racines calculees
*        sommod : sommets du rectangle de recherche effectivement
*                 utilise (en principe il s'agit d'une deformation
*                 legere du rectangle initial somrec)
*                 (une possibilite envisagee, mais non exploitee ici,
*                  a ete d'utiliser ce rectangle a la place de somrec
*                  si la difference de leurs aires n'excede pas 10 %)
*       ----------------------------------------------------------------
      numrac = 0
      nivsub = 1
      call IMPMES(ius,'QUADRI',-1,' ',codret)
      if (codret .ne. 0) goto 60
      write(txtsup,70)  (somrec(i),i=1,4)
      call IMPMES(ius,'QUADRI',-2,'-'//txtsup,codret)
      if (codret .ne. 0) goto 60
      call IMPMES(  0,'QUADRI',-2,'-'//txtsup,codret)
      if (codret .ne. 0) goto 60
*       On tient compte d'une eventuelle execution sans erreur
*       prealable de ce sous-programme
      if (nbsep .ne. 0) then
       sompil = 0
       call CALMUL(nbprec,ks,nivsub,izkdpd,nbsep,absci,sommod,sompil,
     & codret)
       if (codret .eq. 0) goto 30
      endif
*
*       Echec de la decomposition proposee
      do 10 i=1,4
   10  sommod(i) = somrec(i)
*       Recherche du nombre de racines dans le cadre initial, que l'on
*       agrandit eventuellement jusqu'a ce qu'une valeur coherente de
*       la multiplicite soit trouvee.
      call DNBRAC(prmin,nbprec,ks,sommod,nivsub,multi,izkdpd,codret)
      if (codret .ne. 0) goto 40
      write(txtsup,70)  (sommod(i),i=1,4)
      call IMPMES(ius,'QUADRI',-3,'-'//txtsup,codret)
      if (codret .ne. 0) goto 60
      call IMPMES(  0,'QUADRI',-3,'-'//txtsup,codret)
      if (codret .ne. 0) goto 60
      write(txtsup(1:12),'(i12)') multi
      call IMPMES(ius,'QUADRI',-4,'-'//txtsup(1:12),codret)
      if (codret .ne. 0) goto 60
      call IMPMES(  0,'QUADRI',-4,'-'//txtsup(1:12),codret)
      if (codret .ne. 0) goto 60
      do 15 i=1,4
   15  som(i) = sommod(i)
*       S'il y a au plus 2 racines dans ce rectangle, on le traite.
      if (multi .le. 2) goto 25
*
*       Analyse du rectangle (eventuellement) modifie et tentative de
*       separation des racines ; en cas de succes, empilement des
*       rectangles trouves.
      ninter = 10
   20 ninter = ninter + 10
       if (ninter .gt. nbintx) goto 25
       write(txtsup(1:12),'(i12)') ninter
       call IMPMES(0,'QUADRI',-5,'-'//txtsup(1:12),codret)
       call ANALYS(nbprec,som,ninter,nbsep,absci,codret)
       if (codret .ne. 0) goto 55
       sompil = 0
       call CALMUL(nbprec,ks,nivsub,izkdpd,nbsep,absci,sommod,sompil,
     & codret)
      if (codret.ne.0) goto 20
      goto 30
*
*       L'analyse precedente a echoue : derniere chance !
*       Il y a multi racines dans le rectangle modifie.
*       On recherche les racines par subdivision de ce cadre
*       jusqu'a ce qu'il n'y ait plus de cadre a traiter (pile
*       vide) ou que la dimension du cadre soit trop petite.
   25 sompil = 0
      if (multi .ne. 0) then
       call PILEMP(som,nivsub,multi,izkdpd(1),sompil,codret)
      endif
*
*       Boucle de traitement de tous les cadres empiles
   30 if (sompil .gt. 0) then
       call PILDEP(som,nivsub,multi,izkdpd(1),sompil)
*       Sont exclus les cadres situes "au dessous" de la droite y = pimin
*       (en prenant pimin voisin de 0, ceci permet de tenir compte du
*       fait que les racines sont symetriques par rapport a l'axe Ox et
*       de ne calculer que les racines de partie imaginaire positive).
       if (som(4).lt.pimin) goto 30
* Pour suivre le calcul...
*       write(*,70,err=60) (som(i),i=1,4), multi
       call TRCAD(ius,nbpcer,nbprec,pimin,taimai,toler,som,nivsub,
     & multi,izkdpd(1),sompil,numrac,codret)
       if (codret.eq.0 .or. codret.eq.3) goto 30
       if (codret.eq.1) goto 50
       if (codret.eq.2) goto 60
      endif
      goto 90
*
*       Messages d'erreur
   40 nbmes = 6
      goto 80
   50 nbmes = 7
      goto 80
   55 nbmes = 8
      goto 80
   60 nbmes = 9
   80 nbsep = 0
      call IMPMES(  0,'QUADRI',nbmes,' ',codret)
      call IMPMES(ius,'QUADRI',nbmes,' ',codret)
      goto 90
*
   70 format(' [',g10.4,',',g10.4,' ] x [ ',g10.4,',',g10.4,']',i4)
*
   90 return   
      end
************************************************************************
      subroutine RACLAS(nbrac)
      integer nbrac
*  Variables locales
      integer ks, kech, k, m
      complex z
*  Communs
      integer nbracx
      parameter (nbracx = 30)
      integer multip(nbracx)
      complex racine(nbracx)
      common /pilrac/ racine, multip
*       -----------------------------------------------
*       Classement par ordre croissant de partie reelle
*       des racines stockees dans la pile.
*       Arguments d'entree  : nbrac
*       Arguments de sortie :
*        nbrac : nombre de racines dans la pile
*                 --> aucune verification n'est faite
*       -----------------------------------------------
      ks = nbrac
   10 kech = 0
      do 20 k=1,ks-1
       if (real(racine(k)) .le. real(racine(k+1))) goto 20
*       Echange des racines et des multiplicites associees
       kech = k
       z = racine(k)
       racine(k) = racine(k+1)
       racine(k+1) = z
       m = multip(k)
       multip(k) = multip(k+1)
       multip(k+1) = m
   20 continue
      ks = kech
      if (ks .ne. 0) goto 10
      end
************************************************************************
      subroutine RACDEP(numrac,mul,z)
      integer mul, numrac
      complex z
*  Communs
      integer nbracx
      parameter (nbracx = 30)
      integer multip(nbracx)
      complex racine(nbracx)
      common /pilrac/ racine, multip
*       ------------------------------------------------
*       Recuperation d'une racine z avec sa multiplicite
*       mul a partir de la pile de stockage.
*       Arguments d'entree  : numrac
*       Arguments de sortie : mul,z,codret
*        numrac : numero de la racine dans la pile
*                 --> aucune verification n'est faite
*        mul, z : multiplicite et racine renvoyes
*       ------------------------------------------------
      mul = multip(numrac)
      z   = racine(numrac)
      end
************************************************************************
      subroutine RACEMP(mul,z,numrac,codret)
      integer mul, numrac, codret
      complex z
*  Communs
      integer nbracx
      parameter (nbracx = 30)
      integer multip(nbracx)
      complex racine(nbracx)
      common /pilrac/ racine, multip
*       -----------------------------------------------------
*       Stockage d'une nouvelle racine z avec sa multiplicite
*       mul dans la pile de stockage.
*       Arguments d'entree  : mul,z,numrac
*       Arguments de sortie : numrac,codret
*        mul, z : multiplicite et racine a stocker
*        numrac : indice indiquant le sommet de pile
*        codret : code de retour egal a 0 si RAS,
*                 1 si la pile de stockage est pleine.
*       -----------------------------------------------------
      numrac = numrac + 1
      if (numrac .le. nbracx) then
       codret = 0
       multip(numrac) = mul
       racine(numrac) = z
      else
       codret = 1
      endif
      end
************************************************************************
      real function RAYON(z,som)
      complex z
      real som(4)
*  Variables locales
      real d,x,y
*       ----------------------------------------------
*       Determination de la distance min du point z au
*       bord du rectangle de sommets (som(i),i=1,4).
*       Arguments d'entree  : z,som
*       Arguments de sortie : valeur de RAYON
*       ----------------------------------------------
      d = real(z)
      x = min(som(2)-d,d-som(1))
      d = aimag(z)
      y = min(som(4)-d,d-som(3))
      RAYON = min(x,y)
      end
************************************************************************
      subroutine SCIND(nbpcer,som,nivsub,integ,zap,izkdpd,ds2,resul)
      integer nbpcer,nivsub,resul
      real som(4), ds2
      complex integ(*), zap(2), izkdpd(0:1,2)
*  Variables locales
      integer i, mul(2)
      real r(2)
*  Point d'entree
      real RAYON
*       ------------------------------------------------------------
*       On tente de "scinder" une racine en deux, ou plus exactement
*       de determiner si une racine est double ou non.
*       Arguments d'entree  : nbpcer,som,nivsub,integ
*       Arguments de sortie : zap,izkdpd,ds2,resul
*        nbpcer : nombre de points pour le calcul des integrales
*                 sur un cercle
*        som    : sommets du cadre contenant la (les) racine(s)
*        nivsub : niveau courant de subdivision
*        integ  : somme et somme des carres des 2 racines contenues
*                 dans ce cadre
*        zap    : premiere approximation des racines
*        izkdpd : integrales approximations de la multiplicite et de
*                 la somme des racines.
*        ds2    : demi-distance entre les racines
*        resul  : renvoie le bilan des tests, a savoir vaut :
*                  1 si l'on a affaire a 2 racines simples
*                  2 "    "       "    a une racines double
*                  3 en cas d'indecision     
*       ------------------------------------------------------------
*      On utilise ici la somme des carres des racines integ(2)
      zap(2) = sqrt(2*integ(2) - integ(1)*integ(1))
      zap(1) = (integ(1) + zap(2)) * 0.5
      zap(2) = integ(1) - zap(1)
*        On a a priori 2 racines zap(1) et zap(2).
*        On tente maintenant de les separer : calcul des integrales
*        sur un cercle entourant chaque racine.
*        integ(1)/2 est le milieu de [zap(1),zap(2)].
      ds2 = abs(zap(1)-integ(1)*0.5)
      do 10 i=1,2
       r(i) = min(RAYON(zap(i),som),ds2)
       call INCAUC(zap(i),r(i),nbpcer,1,izkdpd(0,i))
   10  call DMULTI(izkdpd(0,i),nivsub,mul(i))
*
*       Bilan :
*       2 racines simples
      if (mul(1).eq.1 .and. mul(2).eq.1) then
       resul = 1
*      Sens du test qui suit : si la racine est double, l'approximation
*      precedente est mauvaise et alors mul(i) = -1.
      elseif (mul(1).lt.0 .and. mul(2).lt.0) then
       resul = 2
      else
*       Indecision
       resul = 3
      endif
      end
************************************************************************
      subroutine SEPRAC(numrac,sommod,nbsep,absci)
      integer numrac,nbsep
      real sommod(*),absci(*)
*  Variables locales
      integer i, multi
      real dmin, dmax, dlim, fract
      complex zim1, zi
*  Initialisation
      parameter (fract = 0.3)
*       ---------------------------------------------------------
*       Separation des racines supposees classees par ordre
*       croissant de partie reelle. Le sous-programme regroupe
*       les racines par paquets et renvoie les abscisses des
*       separateurs.
*       Seuls les separateurs internes sont modifies, les racines
*       etant situees obligatoirement entre les separateurs
*       extremes sommod(1) et sommod(2).
*       Arguments d'entree  : numrac,sommod,nbsep
*       Arguments de sortie : nbsep,absci
*       ---------------------------------------------------------
      if (nbsep .eq. 0) return
      nbsep = 1
      absci(nbsep) = sommod(1)
      if (numrac .gt. 1) then
*       Calcul du min et du max du vecteur difference
       i = 1
       call RACDEP(i,multi,zim1)
       dmin = sommod(2) - sommod(1)
       dmax = 0.
       do 10 i=2,numrac
        call RACDEP(i,multi,zi)
        dlim = real(zi)-real(zim1)
        dmin = min(dmin, dlim)
        dmax = max(dmax, dlim)
   10   zim1 = zi
*       Le test suivant concerne le cas de racines equidistantes
       if (abs(dmax-dmin) .lt. 1.e-6) dmin = 0.
       dlim = dmin+(dmax-dmin)*fract
*       Determination des groupes de racines
       i = 1
       call RACDEP(i,multi,zim1)
       do 20 i=2,numrac
        call RACDEP(i,multi,zi)
        if (real(zi)-real(zim1) .gt. dlim) then
          nbsep = nbsep + 1
          absci(nbsep) = (real(zi)+real(zim1)) * 0.5
        endif
   20   zim1 = zi
      endif
      nbsep = nbsep + 1
      absci(nbsep) = sommod(2)
      end
************************************************************************
      subroutine SOMPAR(jd,jfx,jfy,hx,hy,vdx,vdy,som,ks,sxz,syz)
      integer jd,jfx,jfy, ks
      real hx,hy,vdx,vdy,som(4)
      complex sxz(0:ks),syz(0:ks)
*  Variables locales
      integer j, k
      real r
      complex z1,z2, fz1,fz2
*  Point d'entree
      complex DPZSDZ
*       -----------------------------------------------------------
*       Calcul de sommes partielles intervenant dans l'expression
*       d'une integrale de Cauchy sur un rectangle avec une methode
*       d'integration composee (appele par INCAUR).
*       Arguments d'entree  : jd,jfx,jfy,hx,hy,vdx,vdy,som,ks
*       Arguments de sortie : sxz,syz
*        jd      : indice de debut de la somme
*        jfx,jfy : indices de fin des sommes en x et y
*        hx,hy   : pas en x et y
*        vdx,vdy : abscisses de depart en x et y
*        som     : bornes des intervalles 
*                  ([som(1),som(2)] en x, [som(3),som(4)] en y)
*        ks : exposant de z dans la derniere integrale a calculer
*        sxz  : sommes partielles cherchees en x
*        syz  : sommes partielles cherchees en y
*       -----------------------------------------------------------
      do 5 j=0,ks
       sxz(j) = cmplx(0.)
    5  syz(j) = cmplx(0.)
*
      do 10 j=jd,jfx
       r = vdx + j*hx
       z1 = cmplx(r,som(3))
       z2 = cmplx(r,som(4))
       fz1 = DPZSDZ(z1)
       fz2 = DPZSDZ(z2)
       sxz(0) = sxz(0) + fz1 - fz2
       do 10 k=1,ks
        fz1 = z1*fz1
        fz2 = z2*fz2
   10   sxz(k) = sxz(k) + fz1 - fz2
*
      do 20 j=jd,jfy
       r = vdy + j*hy
       z1 = cmplx(som(2),r)
       z2 = cmplx(som(1),r)
       fz1 = DPZSDZ(z1)
       fz2 = DPZSDZ(z2)
       syz(0) = syz(0) + fz1 - fz2
       do 20 k=1,ks
        fz1 = z1*fz1
        fz2 = z2*fz2
   20   syz(k) = syz(k) + fz1 - fz2
      end
************************************************************************
      subroutine SUBCAD(ius,nbprec,pimin,som,nivsub,multi,integ,sompil,
     & codret)
      integer ius, nbprec, nivsub, multi, sompil, codret
      real pimin, som(4)
      complex integ(*)
*  Variables locales
*       nbsbpx = nombre de subdivisions possibles
*                (en 2 (horiz. ou vert.), en 4, en 3 horiz.)
      integer nptpas, ngdpas, ks, nbscxx, nbsbpx
*       nbscxx = max(nbscx)
      parameter (nptpas = 5,ngdpas = 6, ks = 2, nbscxx = 4, nbsbpx = 4)
      integer i, j, k, igd, ipt, iptpj, mul(nbscxx), typsub, sens
      integer nbscx(nbsbpx), subpos(nbsbpx), ordre(nbsbpx)
      real milieu(2), pas(2), sxy(3,2), sp(4,nbscxx), decal, paspr
      complex izkdpd(0:ks,nbscxx)
      character*100 txtsup
      data nbscx /2, 2, 4, 1/, ordre /1, 3, 2, 4/, decal /1.e-2/
*       ---------------------------------------------------------------
*       Subdivision du cadre de sommets (som(i),i=1,4) qui contient
*       multi racines. Le cadre est subdivise en 2 ou en 4 en empilant
*       chaque sous-cadre contenant au moins une racine.
*       Le decoupage en parties egales peut etre mauvais si le contour
*       est proche d'une racine ; on effectue alors plusieurs essais
*       de decoupage differents.
*       Arguments d'entree  : ius,nbprec,pimin,som,nivsub,multi,integ
*       Arguments de sortie : sompil,codret
*        ius    : numero de l'unite logique du fichier resultat
*        nbprec : nombre de points pour le calcul des integrales
*                 sur un rectangle
*        pimin  : partie imaginaire min en deca de laquelle les
*                 exposants ne sont pas recherches
*        som    : sommets du cadre a considerer
*        nivsub : niveau courant de subdivision
*        multi  : multiplicite des racines contenues dans ce cadre (#0)
*        integ  : somme et somme des carres des racines contenues dans
*                 ce cadre
*        sompil : indice indiquant le sommet de la pile des cadres
*        codret : code de retour egal a 0 si RAS, 1 si la pile est
*              pleine, 2 en cas d'erreur d'ecriture dans le fichier,
*              3 si la subdivision a echoue.
*       ---------------------------------------------------------------
*
*       Determination du type de subdivision binaire (vertical ou
*       horizontal) a utiliser en priorite de maniere a obtenir des
*       sous-cadres a "peu pres carres" ; en cas d'echec on essaie
*       une autre subdivision.
*       Initialisations preliminaires.
      if (som(4)-som(3) .lt. som(2)-som(1)) then
       subpos(1) = 1
      else
       subpos(1) = 2
      endif
      subpos(2) = 3 - subpos(1)
      subpos(3) = 3
      subpos(4) = 4
*
      igd = 2*nptpas*ngdpas
      do 10 i=1,2
       j = 2*subpos(i)
       pas(subpos(i)) = (som(j)-som(j-1)) / igd
       sxy(1,i) =  som(2*i-1)
       sxy(2,i) = (som(2*i-1) + som(2*i)) * 0.5
   10  sxy(3,i) =  som(2*i)
      do 15 i=1,nbscxx
   15  mul(i) = 0
*
*      Test de la frontiere mediane pour la subdivision binaire
*      selectionnee. (subdivision binaire uniquement puisque decouper
*      en 4 echouera si ce premier decoupage a echoue)
*      On n'utilise pas l'axe des x comme frontiere de sous-cadre car
*      les integrales sont imprecises lorsque le contour est proche
*      d'une racine, ce qui arrivera si la racine recherchee est reelle
*      ou si la partie imaginaire est petite.
      typsub = subpos(1)
      if (typsub.ne.2 .or. abs(sxy(2,2)) .gt. decal) then
       call TSTCAD(nbprec,pimin,typsub,nbscx(typsub),ks,nivsub,multi,
     & sxy,sp,mul,izkdpd,codret)
       if (codret.eq.0) goto 90
      endif
*
      do 25 i=1,2
   25  milieu(subpos(i)) = sxy(2,subpos(i))
*
*      Deplacement de la frontiere interne alternativement de part
*      et d'autre de la position mediane et test.
*      Le cadre est decoupe en "tranches" : par rapport au milieu,
*      on teste "loin" (a grand pas) puis on revient (a petit pas),
*      plus "loin" puis on revient, etc...
*      Les differents types de subdivision sont testes dans l'ordre
*      donne par le tableau ordre : on commence par la subdivision
*      binaire selectionnee au debut, puis vient la subdivision en 4.
      do 30 igd=0,ngdpas-2
       j = igd*nptpas
       do 30 ipt=nptpas,1,-1
        iptpj = ipt+j
        do 30 sens=1,-1,-2
         do 30 i=1,2
          typsub = subpos(ordre(i))
          if (typsub .ne. 3) then
           sxy(2,typsub) = milieu(typsub) + sens*iptpj*pas(typsub)
           call TSTCAD(nbprec,pimin,typsub,nbscx(typsub),ks,nivsub,
     &     multi,sxy,sp,mul,izkdpd,codret)
          else
*       Subdivision de type 3, soit en 4 sous-cadres : deplacement du
*       centre du reticule selon chacune des 2 diagonales successivement.
*       Premiere diagonale :
           do 40 k=1,2
   40       sxy(2,k) = milieu(k) + sens*iptpj*pas(k)
           call TSTCAD(nbprec,pimin,typsub,nbscx(typsub),ks,nivsub,
     &     multi,sxy,sp,mul,izkdpd,codret)
           if (codret.eq.0) goto 90
*       Deuxieme diagonale :
           sxy(2,2) = milieu(2) - sens*iptpj*pas(2)
           call TSTCAD(nbprec,pimin,typsub,nbscx(typsub),ks,nivsub,
     &     multi,sxy,sp,mul,izkdpd,codret)
          endif
          if (codret.eq.0) goto 90
   30 continue
*       Derniere tentative : si la forme du rectangle est telle que
*       dy > dx, on essaie de decouper horizontalement en prenant
*       des rectangles centres verticalement (du fait qu'au depart,
*       les rectangles sont le plus souvent de ce type).
      k = 2
      sxy(2,k) = milieu(k) + (som(2)-som(1))*0.5
      j = max(3,ngdpas)
      paspr = (som(4) - sxy(2,k)) / j
      if (subpos(1).ne.1 .and. paspr.gt.0.) then
       typsub = subpos(ordre(4))
       sxy(2,k) = sxy(2,k) - 2*paspr
       do 50 igd=1,j+2
        call TSTCAD(nbprec,pimin,typsub,nbscx(typsub),ks,nivsub,
     &  multi,sxy,sp,mul,izkdpd,codret)
        if (codret.eq.0) goto 90
   50   sxy(2,k) = sxy(2,k) + paspr
      endif
*       Echec total...
      goto 120
*
*      On empile les sous-cadres qui contiennent des racines
   90 do 100 i=1,nbscx(typsub)
       if (mul(i).ne.0) then
        call PILEMP(sp(1,i),nivsub,mul(i),izkdpd(1,i),sompil,codret)
        if (codret.ne.0) goto 110
       endif
  100 continue
CC
CC       if (ipt.eq.nptpas) then
CC        write(44,*) MOU, 'grand', igd, iptpj
CC       else
CC        write(44,*) MOU, 'petit', igd, iptpj
CC       endif
CC
*
  110 return
*
  120 write(txtsup,140) (som(i),i=1,4), multi,integ(1)
      call IMPMES(ius,'SUBCAD',1,' '//txtsup,codret)
      if (codret .ne. 0) goto 130
      return
*
  130 codret = 2
*
  140 format('[',g10.4,',',g10.4,' ] x [ ',g10.4,',',g10.4,'], ',i2,
     & ', ',g20.12e3,sp,g20.12e3,' *i')
      end
************************************************************************
      subroutine SUICAL(ius,multi,zap,z,r,codret)
      integer ius,multi,codret
      complex zap,z
      real r
      character*46 txtsup
*       -----------------------------------------------------------
*       Impression sur le fichier associe a l'unite logique ius
*       de la racine z et de sa valeur approchee zap.
*       r est le rayon utilise pour le calcul de l'integrale de
*       contour circulaire.
*       Arguments d'entree  : ius,multi,zap,z,r
*       Arguments de sortie : codret
*        ius    : numero de l'unite logique du fichier resultat
*        codret : code de retour egal a 0 si RAS, 2 en cas d'erreur
*                 d'ecriture dans le fichier.
*       -----------------------------------------------------------
      codret = 0
      if (multi .gt. 1) then
       write(txtsup(1:2),'(i2)') multi
       call IMPMES(ius,'SUICAL',-1,'-'//txtsup(1:2),codret)
       if (codret .ne. 0) goto 10
      endif
      write(txtsup,20) zap
      call IMPMES(ius,'SUICAL',-2,'-'//txtsup,codret)
      if (codret .ne. 0) goto 10
      write(txtsup,20) z
      call IMPMES(ius,'SUICAL',-3,'-'//txtsup,codret)
      if (codret .ne. 0) goto 10
      write(txtsup(1:11),'(g11.4)') r
      call IMPMES(ius,'SUICAL',-4,'-'//txtsup(1:11),codret)
      if (codret .ne. 0) goto 10
      return
*
   10 codret = 2
*
   20 format(1x,g20.12e3,1x,sp,g20.12e3,' *i')
      end
************************************************************************
      subroutine TRCAD(ius,nbpcer,nbprec,pimin,taimai,toler,som,nivsub,
     & multi,integ,sompil,numrac,codret)
      integer ius, nbpcer,nbprec, nivsub, multi, sompil, numrac, codret
      real pimin,taimai,toler,som(4)
      complex integ(*)
*  Variables locales
      integer i, ks, nsubut, codimp
      real d, ray
      parameter (ks = 1)
      character*100 txtsup
      complex izkdpd(0:ks), zrac
*       ---------------------------------------------------------------
*       Traitement du cadre de sommets (som(i),i=1,4) qui contient
*       multi racines.
*       . Si multi = 1 ou si la precision est atteinte :
*         - si multi=1, on stocke la racine trouvee,
*         - sinon, arret : limite de subdivision atteinte. La limite
*           est definie par taimai et mesure le plus petit cote du
*           rectangle som caracterisant la maille de la subdivision.
*           On calcule une approximation de la racine multiple.
*       . Sinon, si multi=2, on tente de separer les 2 racines.
*       . Sinon on subdivise le cadre en empilant chaque sous-cadre
*         contenant au moins une racine.
*       Arguments d'entree  : ius,nbpcer,nbprec,pimin,taimai,toler,
*                             som,nivsub,multi,integ,sompil,numrac
*       Arguments de sortie : sompil,numrac,codret
*        ius    : numero de l'unite logique du fichier resultat
*        nbpcer,nbprec : nombre de points pour le calcul des
*                      integrales sur un cercle et sur un rectangle
*        pimin  : partie imaginaire min en deca de laquelle les
*                 exposants ne sont pas recherches
*        taimai : taille min de la maille rectangulaire lors du
*                 decoupage du plan pour la recherche des racines
*        toler  : precision requise pour le calcul des exposants
*        som    : sommets du cadre a considerer
*        nivsub : niveau courant de subdivision
*        multi  : multiplicite des racines contenues dans ce cadre
*        integ  : somme et somme des carres des racines contenues dans
*                 ce cadre
*        sompil : indice indiquant le sommet de la pile des cadres
*        numrac : indice indiquant le sommet de la pile de stockage
*                 des racines calculees
*        codret : code de retour egal a 0 si RAS, 1 si la pile est
*              pleine, 2 en cas d'erreur d'ecriture dans le fichier,
*              3 si la subdivision a echoue.
*       ---------------------------------------------------------------
      codret = 0
      d = min(som(2)-som(1),som(4)-som(3))
      if (multi.eq.1 .or. d.lt.taimai) then
*
*       Calcul d'une meilleure approximation par une integrale
*       sur un cercle contenu dans le cadre.
       zrac = integ(1)
       call ESTIPR(nbpcer,toler,som,nivsub,ks,multi,zrac,ray,izkdpd,
     & nsubut,codret)
*
*       Impression des resultats
       if (multi.eq.1) then
*        Une seule racine dans le cadre
        if (codret .eq. 0) then
         if (aimag(zrac) .ge. pimin) then
          call SUICAL(ius,multi,integ(1),zrac,ray,codret)
          call RACEMP(multi,zrac,numrac,codret)
         endif
        else
         if (d.lt.taimai) then
*          On a atteint la limite
          write(txtsup,60) (som(i),i=1,4), multi,integ(1)
          call IMPMES(ius,'TRCAD',1,' '//txtsup,codimp)
          if (codimp .ne. 0) goto 50
          codret = 3
         else
*          On peut encore reduire la taille du cadre
          call SUBCAD(ius,nbprec,pimin,som,nivsub+1,multi,integ,sompil,
     &    codret)
         endif
        endif
       else
*        La taille min de la maille est atteinte
        write(txtsup(1:42),'(2(1x,g20.12e3))') d, taimai
        call IMPMES(ius,'TRCAD',2,' '//txtsup(1:42),codimp)
        if (codimp .ne. 0) goto 50
        if (codret .ne. 0) then
         if (ray .lt. 0.) then
*          integ(1)/multi est a l'exterieur du rectangle som ce qui
*          conduit a un rayon ray < 0.
*          On essaie de recuperer en prenant le centre du rectangle,
*          puis eventuellement l'entier le plus proche.
          zrac = multi * cmplx(som(1)+som(2), som(3)+som(4)) / 2
          call ESTIPR(nbpcer,toler,som,nivsub,ks,multi,zrac,ray,izkdpd,
     &    nsubut,codret)
          if (codret.ne.0 .and. som(3)*som(4).lt.0.) then
           zrac = multi * cmplx(nint((som(1)+som(2))/2))
           call ESTIPR(nbpcer,toler,som,nivsub,ks,multi,zrac,ray,izkdpd,
     &     nsubut,codret)
          endif
         endif
         if (codret .ne. 0) then
          zrac = integ(1)
          write(txtsup,60) (som(i),i=1,4), multi,integ(1)
          call IMPMES(ius,'TRCAD',3,' '//txtsup,codimp)
          if (codimp .ne. 0) goto 50
         endif
        endif
        zrac = zrac / multi
        call SUICAL(ius,multi,integ(1) / multi,zrac,ray,codret)
        call RACEMP(multi,zrac,numrac,codret)
       endif
*
      else
       if (multi.eq.2) then
*       Separation des 2 racines
        call TRCAM2(ius,nbpcer,pimin,toler,som,nivsub,integ,
     &              sompil,numrac,codret)
        if (codret .eq. 3) then
         call SUBCAD(ius,nbprec,pimin,som,nivsub+1,multi,integ,sompil,
     &   codret)
        endif
       else
*       Subdivision du cadre
        call SUBCAD(ius,nbprec,pimin,som,nivsub+1,multi,integ,sompil,
     &  codret)
       endif
      endif
      return
*
   50 codret = 2
*
   60 format('[',g10.4,',',g10.4,' ] x [ ',g10.4,',',g10.4,'], ',i2,
     & ', ',g20.12e3,sp,g20.12e3,' *i')
      end
************************************************************************
      subroutine TRCAM2(ius,nbpcer,pimin,toler,som,nivsub,integ,
     & sompil,numrac,codret)
      integer ius, nbpcer,nivsub,sompil,numrac,codret
      real pimin,toler,som(4)
      complex integ(*)
*  Variables locales
      integer i, ks, mul, nsubut, resul
      parameter (ks = 2)
      complex zap(2), zpre, izkdpd(0:1,2), intcmp(0:ks)
      real ray(2), ds2, sompr(4)
*       ------------------------------------------------------------
*       Traitement du cas ou la multiplicite vaut 2 : il s'agit soit
*       d'une racine double, soit de 2 racines situees dans le meme
*       rectangle de sommets som.
*       On essaie tout d'abord de separer les racines. Trois cas se
*       presentent alors :
*        1. la racine est jugee double : on calcule une meilleure
*           approximation des integrales et on essaie a nouveau de
*           separer. Si la racine est a nouveau jugee double, on la
*           stocke, sinon on empile le dernier cadre utile pour
*           traitement ulterieur. En sortie, codret = 0.
*        2. il s'agit de 2 racines simples : on en calcule une
*           meilleure approximation. Si ce calcul ne pose pas de
*           probleme on les stocke, sinon on sort avec codret = 3.
*        3. il y a indecision : on sort avec codret = 3.
*       Arguments d'entree  : ius,nbpcer,pimin,toler,som,
*                             nivsub,sompil,integ,numrac
*       Arguments de sortie : sompil,numrac,codret
*        ius    : numero de l'unite logique du fichier resultat
*        nbpcer : nombre de points pour le calcul des integrales
*                 sur un cercle
*        pimin  : partie imaginaire min en deca de laquelle les
*                 exposants ne sont pas recherches
*        toler  : precision requise pour le calcul des exposants
*        som    : sommets du cadre contenant la (les) racine(s)
*        nivsub : niveau courant de subdivision
*        integ  : somme et somme des carres des 2 racines contenues
*                 dans ce cadre
*        sompil : indice indiquant le sommet de la pile des cadres
*        numrac : indice indiquant le sommet de la pile de stockage
*                 des racines calculees
*        codret : code de retour egal a 0 si RAS, 1 si une des piles
*                 est pleine, 2 en cas d'erreur d'ecriture dans le
*                 fichier, 3 si la separation a echoue.
*       ------------------------------------------------------------
*
      call SCIND(nbpcer,som,nivsub,integ,zap,izkdpd,ds2,resul)
*
      if (resul .eq. 2) then
*        Cas d'une racine pretendue double : calcul d'une meilleure
*        approximation, puis si le cadre qui la contient n'est pas situe
*        au dessous de pimin, on effectue un zoom pour verifier s'il
*        s'agit bien d'une racine double : si tel est le cas, on stocke
*        la racine, sinon on empile le cadre pour traitement par TRCAD
       mul = 2
       zpre = integ(1)
       call ESTIPR(nbpcer,toler,som,nivsub,ks,mul,zpre,ray(1),intcmp,
     & nsubut,codret)
       if (codret .eq. 0) then
        zpre = zpre / mul
        sompr(4) = aimag(zpre) + ray(1)
        if (sompr(4) .gt. pimin) then
         sompr(1) = real(zpre) - ray(1)
         sompr(2) = real(zpre) + ray(1)
         sompr(3) = aimag(zpre) - ray(1)
         call SCIND(nbpcer,sompr,nsubut,intcmp(1),zap,izkdpd,ds2,resul)
         if (resul .eq. 2) then
*          Il y a confirmation : la racine est double.
*          On conserve l'approximation issue de ESTIPR
          call SUICAL(ius,mul,integ(1) / mul,zpre,ray(1),codret)
          call RACEMP(mul,zpre,numrac,codret)
         else
          call PILEMP(sompr,nsubut,mul,intcmp(1),sompil,codret)
         endif
        endif
       else
        codret = 3
       endif
*
      elseif (resul .eq. 1) then
*        Cas de 2 racines : calcul d'une meilleure approximation
       mul = 1
       ds2 = ds2 * .707106781
       do 20 i=1,2
        zpre = izkdpd(1,i)
        sompr(1) = max(som(1), real(zpre) - ds2)
        sompr(2) = min(som(2), real(zpre) + ds2)
        sompr(3) = max(som(3), aimag(zpre) - ds2)
        sompr(4) = min(som(4), aimag(zpre) + ds2)
*        Si le test suivant est verifie, celui de la boucle qui suit
*        le sera a fortiori.
        if (sompr(4) .lt. pimin) goto 20
        call ESTIPR(nbpcer,toler,sompr,nivsub,1,mul,zpre,ray(i),intcmp,
     &  nsubut,codret)
        if (codret .ne. 0) then
         codret = 3
         goto 29
        endif
   20  izkdpd(1,i) = zpre
       do 25 i=1,2
        if (aimag(izkdpd(1,i)) .lt. pimin) goto 25
        call SUICAL(ius,mul,zap(i),izkdpd(1,i),ray(i),codret)
        call RACEMP(mul,izkdpd(1,i),numrac,codret)
        if (codret .ne. 0) goto 30
   25  continue
   29  continue
*
      else
*       Indecision
       codret = 3
      endif
   30 return
*
      end
************************************************************************
      subroutine TSTCAD(nbprec,pimin,typsub,nbscx,ks,nivsub,multi,sxy,
     & som,mul,izkdpd,codret)
      integer typsub,nbprec,nbscx,ks,nivsub,multi, mul(*), codret
      real pimin,sxy(3,*), som(4,*)
      complex izkdpd(0:ks,*)
*  Variables locales
      integer numsc, smul, i
*       -------------------------------------------------------------
*       Test de la subdivision d'un cadre : determination du nombre
*       de racines situees a l'interieur de chaque sous-cadre et
*       calcul des integrales de contour necessaires.
*       Arguments d'entree  : nbprec,pimin,typsub,nbscx,ks,nivsub,
*                             multi,sxy
*       Arguments de sortie : som,mul,izkdpd,codret
*        nbprec : nombre de points pour le calcul des integrales
*                 sur un rectangle
*        pimin  : partie imaginaire min en deca de laquelle les
*                 exposants ne sont pas recherches
*        typsub : type de la subdivision du cadre de depart
*        nbscx  : nombre de sous-cadres de la subdivision
*        ks     : nombre d'integrales a calculer
*        nivsub : niveau courant de subdivision
*        multi  : multiplicite des racines contenues dans le cadre
*                 de depart
*        sxy    : tableau contenant les abscisses et ordonnees
*                 definissant le decoupage
*        som    : sommets des sous-cadres (xmin, xmax, ymin, ymax)
*                 definissant la subdivision
*        mul    : multiplicite des racines contenues dans chaque
*                 sous-cadre
*        izkdpd : integrales de contour sur le bord des sous-cadres
*        codret : code de retour egal a 0 en cas de succes, ie si
*                 les calculs ont pu etre menes a bien dans chaque
*                 sous-cadre explore, 3 en cas d'echec.
*       -------------------------------------------------------------
      codret = 0
      do 10 numsc=1,nbscx
   10  mul(numsc) = 0
      smul = 0
      do 20 numsc=1,nbscx
       call NSSCAD(sxy(1,1),sxy(1,2),numsc,typsub,som(1,numsc))
       if (som(4,numsc) .lt. pimin) goto 20
       call INCAUR(som(1,numsc),nbprec,ks,izkdpd(0,numsc))
       call DMULTI(izkdpd(0,numsc),nivsub,mul(numsc))
CC          WRITE(*,'(a,4f6.3,2e21.14,i4)') 'TSTCAD',
CC     &    (som(i,numsc),i=1,4),izkdpd(0,numsc),mul(numsc)
       if (mul(numsc).lt.0 .or. mul(numsc).gt.multi) goto 30
       smul = smul + mul(numsc)
*      Test pour accelerer
       if (smul .eq. multi) return
   20 continue
*      Test de coherence
       if (smul .le. multi) return
*
   30 codret = 3
      end
************************************************************************
      subroutine TSTCOT(nbprec,num,pas,limite,nivsub,som,ok)
      integer nbprec, num, nivsub
      real pas, limite, som(4)
      logical ok
*  Variables locales
      integer ks, multi
      parameter (ks=0)
      real eps, valim
      complex z
*       ------------------------------------------------------------
*       Test d'un cote d'un rectangle. On calcule la multiplicite
*       dans une zone rectangulaire longiligne bordant le cote : on
*       decale eventuellement cette zone jusqu'a en trouver une
*       exempte de racine. Le decalage est limite de sorte qu'il y
*       a echec au bout d'un certain nombre d'essais infructueux.
*       Arguments d'entree  : nbprec,num,pas,limite,nivsub
*       Arguments de sortie : som,ok
*        nbprec : nombre de points pour le calcul des
*                 integrales sur un rectangle
*        num    : numero du sommet de reference pour le decalage
*                 (les coordonnes num et num+1 sont modifiees)
*        pas    : quantite utilisee pour le decalage
*        limite : valeur a ne pas depasser au cours du decalage
*        nivsub : niveau courant de subdivision
*        som    : en entree : sommets du cadre a tester
*                             (xmin, xmax, ymin, ymax)
*                 en sortie : cadre eventuellement translate
*        ok     : booleen FAUX si la recherche a echoue, VRAI sinon
*                 et dans ce cas la multiplicite est nulle dans sp.
*       ------------------------------------------------------------
      ok = .false.
      if (pas .lt. 0.) then
       valim = limite
       eps = 1.
      else
       valim = -(limite-pas)
       eps = -1.
      endif
*
   10 call INCAUR(som,nbprec,ks,z)
      call DMULTI(z,nivsub,multi)
CC       WRITE(*,'(a,4f6.3,2e21.14,i4)') 'TSTCOT', som, z, multi
      if (multi.eq.0) goto 20
*      Decalage pour rechercher une zone sans racine
      som(num) = som(num) + pas
      som(num+1) = som(num+1) + pas
      if (som(num)*eps .gt. valim) goto 10
      return
*
   20 ok = .true.
      end
**************************** Fin du fichier ****************************
