************************************************************************
*  Calcul d'exposants de singularites en elasticite anisotrope.
*  Yvon Lafranche, Universite de Rennes 1.
*  Voir documentation a l'adresse :
*        http://perso.univ-rennes1.fr/yvon.lafranche/exsiel/
************************************************************************
************************************************************************
* ACSUIV : poursuite des actions a realiser
* AFFQPM : affichage des vecteurs de base de la solution
* AFFRDS : affichage des racines du determinant du symbole
* CALADI : calcul de alpha (direct)
* CALALP : calcul de alpha
* CALBAS : calcul de la base de l'espace des solution
* CALCDS : calcul des coefficients du determinant
* CALQPM : calcul des vecteurs qplus et qmoins
* CALVEC : calcul du vecteur colonne, de norme max, de Mad(alpha)
* EXSIEL : module maitre : pilotage des differentes phases du
*          calcul des exposants de singularites
* MAJTC  : mise a jour des coefficients des matrices de rigidite
* MATCOF : calcul de la matrice des cofacteurs associee au symbole
* PPSING : programme principal
* PR3PD2 : produit de 3 polynomes de degre 2
* RCAUTO : recherche automatique des exposants de singularites
* RCMANU : recherche manuelle des exposants de singularites
* ROTFIB : calcul des exposants de sing. avec rotation des materiaux
* ROTREF : calcul des exposants de sing. pour une configuration mobile
* SUIVAR : calcul des exposants de sing. avec suivi d'une arete courbe
* SYMBOL : calcul du symbole
* VARCO  : calcul des exposants de sing. avec variation d'un coefficient
* VARGEO : calcul des exposants de sing. pour une geometrie variable
************************************************************************
      subroutine ACSUIV(unlo,numrac,action,parcou,codret)
      integer unlo(*),numrac,action(2),codret
      real parcou
*  Variables locales
      integer i, multi
      complex z
*       -----------------------------------------------------------
*       Poursuite eventuelle des actions a realiser.
*       Arguments d'entree  : unlo,numrac,action,parcou
*       Arguments de sortie : codret
*        unlo   : numeros des unites logiques des fichiers ou
*                 sont imprimes les resultats
*        numrac : indice indiquant le sommet de la pile de stockage
*                 des racines calculees
*        action : caracterise l'action a realiser
*        parcou : parametre courant pilotant le calcul
*        codret : code de retour egal a 0 si RAS, #1 sinon.
*       -----------------------------------------------------------
      if (action(2) .ge. 1) then
       do 10 i=1,numrac
        call RACDEP(i,multi,z)
        call CALKER(unlo,z,multi,action,parcou,codret)
   10  continue
      endif
*
      return
      end
************************************************************************
      subroutine AFFQPM(ius,dim,qplus,codret)
      integer ius,dim,codret
      complex qplus(dim,dim)
*  Variables locales
      integer j,k
      character*2 txtsup
*       ---------------------------------------------------
*       Affichage des vecteurs de base de la solution.
*
*       Arguments d'entree  : ius,dim,qplus
*       Arguments de sortie : codret
*        ius    : unite logique du fichier de sortie
*        dim    : dimension de l'espace
*        qplus  : vecteurs de base correspondant a M+
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       ---------------------------------------------------
      codret = 0
      do 10 j=1,dim
       write(txtsup,'(i2)') j
       call IMPMES(ius,'AFFQPM',-1,'-'//txtsup,codret)
       do 10 k=1,dim
   10   write(ius,30,err=20) qplus(k,j)
      return
*
   20 codret = 1
   30 format(1x,g23.15e3,1x,sp,g23.15e3,' *i')
      end
************************************************************************
      subroutine AFFRDS(ius,dim,alpha,codret)
      integer ius,dim,codret
      complex alpha(dim)
*  Variables locales
      integer j
*       --------------------------------------------------
*       Affichage des racines du determinant du symbole et
*       recherche d'une eventuelle racine multiple.
*
*       Arguments d'entree  : ius,dim,alpha
*       Arguments de sortie : codret
*        ius    : unite logique du fichier de sortie
*        dim    : dimension de l'espace
*        alpha  : racines du determinant du symbole M+
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       --------------------------------------------------
      codret = 0
      call IMPMES(ius,'AFFRDS',-1,' ',codret)
      if (codret.ne.0) goto 40
      do 10 j=1,dim
   10 write(ius,50,err=40) j, alpha(j), abs(alpha(j))
      do 20 j=2,dim
       if (alpha(j) .eq. alpha(1)) then
        call IMPMES(ius,'AFFRDS',2,' ',codret)
        if (codret.ne.0) goto 40
        call IMPMES(  0,'AFFRDS',2,' ',codret)
        goto 40
       endif
   20 continue
      return
*
   40 codret = 1
   50 format('ALPHA',i2,' = ',g23.15e3,1x,sp,g23.15e3,' *i',ss,
     & 5x,g15.8e3)
      end
************************************************************************
      subroutine CALADI(dim,cc,alpha)
      integer dim
      real cc(*)
      complex alpha(dim)
*  Variables locales
      real gamma, bp, beta
*       ---------------------------------------------------------------
*       Calcul des racines alpha des determinants des symboles M+ et M-
*       ou M+(al) = M(al+1, i(al-1)) et M-(al) = M(al+1, i(1-al)), en
*       dimension 2 dans le cas orthotrope axisymetrique.
*       --> Les racines de M- ne sont pas calculees car ce sont les
*           conjuguees des racines de M+.
*       Arguments d'entree  : dim,cc
*       Arguments de sortie : alpha
*        dim    : dimension de l'espace
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*        alpha  : racines du determinant du symbole M+
*       ---------------------------------------------------------------
      gamma = cc(2) + cc(6)
      beta  = cc(1) - cc(6)
      bp = 1.0 +  8.0 * cc(1) * cc(6) / (beta*beta - gamma*gamma)
*        L'ellipticite impose que abs(bp) > 1.
      if (bp .gt. 0.0) then
       beta = 1.0 / (bp + sqrt(bp*bp - 1.0))
       beta = sqrt(beta)
       alpha(1) = cmplx( beta, 0.0)
       alpha(2) = cmplx(-beta, 0.0)
      else
       beta = 1.0 / (bp - sqrt(bp*bp - 1.0))
       beta = sqrt(-beta)
       alpha(1) = cmplx(0.0, beta)
       alpha(2) = cmplx(0.0,-beta)
      endif
      end
************************************************************************
      subroutine CALALP(ius,dim,cc,alpha,codret)
      integer ius, dim, codret
      real cc(*)
      complex alpha(dim)
*  Variables locales
      integer nx, n, j,k
      parameter (nx = 6)
      real a(0:nx), ap(0:nx), b(0:nx)
      complex z(nx), icomp
      character*12 txtsup
*       ---------------------------------------------------------------
*       Calcul des racines alpha des determinants des symboles M+ et M-
*       ou M+(al) = M(al+1, i(al-1)) et M-(al) = M(al+1, i(1-al)).
*       --> Les racines de M- ne sont pas calculees car ce sont les
*           conjuguees des racines de M+.
*       Arguments d'entree  : ius,dim,cc
*       Arguments de sortie : alpha,codret
*        ius    : unite logique du fichier de sortie
*        dim    : dimension de l'espace
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*        alpha  : racines du determinant du symbole M+
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       ---------------------------------------------------------------
      n = 2*dim
*      Le calcul est fait a partir du symbole M(1,x) dont le
*      determinant est un polynome de degre 2*dim a coefficients reels
      call CALCDS(cc,dim,ap,a)
*      Calcul des racines de ce polynome
      call BERSTO(n,a,ap,b,z,codret)
      if (codret .ne. 0) then
       call IMPMES(ius,'CALALP',1,' ',codret)
       if (codret.ne.0) goto 40
       call IMPMES(  0,'CALALP',1,' ',codret)
       goto 40
      endif
*      On selectionne les racines dont la partie imaginaire est < 0.
      k = 0 
      do 10 j=1,n
       if (aimag(z(j)) .lt. 0.) then
        k = k + 1
        alpha(k) = z(j)
       endif 
   10 continue
      if (k .ne. dim) then
       call IMPMES(ius,'CALALP',2,' ',codret)
       if (codret.ne.0) goto 40
       call IMPMES(  0,'CALALP',2,' ',codret)
       goto 40
      endif
*      Raffinement de la localisation par la methode de Newton, puis
*      on applique la transformation de Cayley inverse permettant
*      d'obtenir les racines cherchees dans le disque unite.
*deb   WRITE(*,*)
*deb   WRITE(*,*) 'DOMAINE ',I
      icomp = cmplx(0.,1.)
      do 20 j=1,dim
*deb   WRITE(*,*) 'RACINE AVANT RAFF ',alpha(j)
       call RACPCR(n,a,ap,alpha(j),codret)
       if (codret .ne. 0) then
        write(txtsup,'(i12)') j
        call IMPMES(ius,'CALALP',3,'-'//txtsup,codret)
        if (codret.ne.0) goto 40
        call IMPMES(  0,'CALALP',3,'-'//txtsup,codret)
       endif
*deb   WRITE(*,*) 'RACINE APRES RAFF ',alpha(j)
       alpha(j) = (icomp+alpha(j)) / (icomp-alpha(j))
*deb   WRITE(*,*) 'RACINE APRES TR CAYLEY ',alpha(j)
   20 continue
*
   40 return
      end
************************************************************************
      subroutine CALBAS(ius,dim,nbcxx,nbdom,contx,cc,mpm,mad,alpha,
     & qplus,codret)
      integer ius,dim,nbcxx,nbdom,contx(nbdom),codret
      real cc(nbcxx,nbdom)
      complex mpm(dim,dim), mad(dim,dim)
      complex alpha(dim,nbdom), qplus(dim,dim,nbdom)
*  Variables locales
      integer i, cmu(2:3)
      character*2 txtsup
*  Initialisation
      data cmu /6, 21/
*       --------------------------------------------------------------
*       Calcul du terme de la base de l'espace des solutions associees
*       a chacun des domaines, qui ne depend que des racines du
*       determinant du symbole.
*       Ce calcul ne fait pas intervenir la geometrie des domaines.
*       Arguments d'entree  : ius,dim,nbcxx,nbdom,contx,cc
*       Arguments de sortie : alpha,qplus,codret
*       Tableaux de travail : mpm,mad
*        ius    : unite logique du fichier de sortie
*        dim    : dimension de l'espace
*        nbcxx  : nombre max de coefficients (1ere dimension de cc)
*        nbdom  : nombre de domaines
*        contx  : contexte du calcul pour chaque domaine
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*        alpha  : racines du determinant du symbole M+ ou combinaison
*                 de coefficients dans le cas isotrope
*        qplus  : vecteurs correspondant a M+ intervenant dans
*                 l'expression de la base
*        codret : code de retour valant 0 si RAS, 1 sinon.
*
*       --> alpha et qplus ne sont pas utilises dans les domaines ou
*           le materiau est isotrope. C'est pourquoi on place dans la
*           partie reelle de alpha(1,.) et alpha(2,.) les valeurs
*           lambda + mu et lambda + 3*mu respectivement (lambda et mu
*           sont les coefficient de Lame), qui seront utilisees dans
*           le calcul de la base de l'espace des solutions.
*       --------------------------------------------------------------
*
      call IMPMES(ius,'CALBAS',-1,' ',codret)
      if (codret.ne.0) goto 50
*
      do 10 i=1,nbdom
       write(txtsup,'(i2)') i
       call IMPMES(ius,'CALBAS',-2,'-'//txtsup,codret)
       if (contx(i) .eq. 1) then
        alpha(1,i) = cmplx(cc(2,i)+  cc(cmu(dim),i), 0.)
        alpha(2,i) = cmplx(cc(2,i)+3*cc(cmu(dim),i), 0.)
        call IMPMES(ius,'CALBAS',-3,' ',codret)
        if (codret.ne.0) goto 50
       else
*        Calcul des racines alpha
*        -------------------------
        if (dim.eq.2 .and. contx(i).le.2) then
         call CALADI(dim,cc(1,i),alpha(1,i))
        else
         call CALALP(ius,dim,cc(1,i),alpha(1,i),codret)
         if (codret .ne. 0) goto 50
        endif
        call AFFRDS(ius,dim,alpha(1,i),codret)
        if (codret .ne. 0) goto 50
*
*        Calcul des vecteurs de base de la solution
*        ------------------------------------------
        call CALQPM(dim,cc(1,i),alpha(1,i),mpm,mad,qplus(1,1,i))
        call AFFQPM(ius,dim,qplus(1,1,i),codret)
       endif
   10 continue
      call SEPARR(ius,codret)
*
   50 return
      end
************************************************************************
      subroutine CALCDS(cc,dim,t,r)
      integer dim
      real cc(*), t(0:2*dim), r(0:2*dim)
*  Variables locales
      integer i, deg
      real a(0:2), b(0:2), c(0:2), d(0:2), e(0:2), f(0:2)
*       -------------------------------------------------
*       Calcul des coefficients du determinant du symbole
*       M(1,x) x dans R, en dimension dim (= 2 ou 3).
*       Arguments d'entree  : cc,dim
*       Arguments de sortie : r
*       Tableau de travail  : t
*        cc  : coefficients de la matrice de rigidite
*        dim : dimension de l'espace
*        t   : tableau de travail utilise en dimension 3
*        r   : coefficients du polynome de degre 2*dim
*              (r(i) coeff de x**i, i=0,2*dim)
*       -------------------------------------------------
      deg = 2*dim
      if (dim.eq.2) then
       r(0) = cc(1)*cc(6) - cc(3)*cc(3)
       r(1) = 2.*(cc(1)*cc(5) - cc(2)*cc(3))
       r(2) = cc(1)*cc(4) - cc(2)*cc(2) + 2.*(cc(3)*cc(5)-cc(2)*cc(6))
       r(3) = 2.*(cc(3)*cc(4) - cc(2)*cc(5))
       r(4) = cc(4)*cc(6) - cc(5)*cc(5)
      else
       a(0) = cc(1)
       a(1) = 2*cc(6)
       a(2) = cc(21)
       b(0) = cc(6)
       b(1) = cc(2) + cc(21)
       b(2) = cc(11)
       c(0) = cc(5)
       c(1) = cc(4) + cc(20)
       c(2) = cc(18)
       d(0) = cc(21)
       d(1) = 2*cc(11)
       d(2) = cc(7)
       e(0) = cc(20)
       e(1) = cc(10) + cc(18)
       e(2) = cc(9)
       f(0) = cc(19)
       f(1) = 2*cc(17)
       f(2) = cc(16)
       call PR3PD2(a,d,f,r)
       call PR3PD2(b,c,e,t)
       do 10 i=0,deg
   10   r(i) = r(i) + 2*t(i)
       call PR3PD2(a,e,e,t)
       do 20 i=0,deg
   20   r(i) = r(i) - t(i)
       call PR3PD2(b,b,f,t)
       do 30 i=0,deg
   30   r(i) = r(i) - t(i)
       call PR3PD2(c,c,d,t)
       do 40 i=0,deg
   40   r(i) = r(i) - t(i)
      endif
*      Normalisation des coefficients (r(deg) est non nul)
      do 50 i=0,deg-1
   50  r(i) = r(i) / r(deg)
      r(deg) = 1.
      end
************************************************************************
      subroutine CALQPM(dim,cc,alpha,mpm,mad,qplus)
      integer dim
      real cc(*)
      complex alpha(dim), mpm(dim,dim), mad(dim,dim)
      complex qplus(dim,dim)
*  Variables locales
      integer j
      complex icomp, un, d1, d2
*       -------------------------------------------------------------
*       Calcul des vecteurs qplus et qmoins intervenant dans
*       l'expression de la base de l'espace des solutions, qplus
*       correspondant a M+ et qmoins a M-.
*       Seul qplus est calcule, qmoins en etant le conjugue.
*       Arguments d'entree  : dim,cc,alpha
*       Arguments de sortie : qplus
*       Tableaux de travail : mpm,mad
*        dim    : dimension de l'espace
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*        alpha  : racines du determinant du symbole M+
*        mpm,mad : stockage temporaire du symbole et de la matrice
*                  des cofacteurs
*        qplus  : vecteurs recherches correspondant a M+
*       -------------------------------------------------------------
      icomp = cmplx(0.,1.)
      un = cmplx(1.)
      do 10 j=1,dim
       d1 = alpha(j) + un
       d2 = icomp * (alpha(j) - un)
       call CALVEC(d1,d2,cc,dim,mpm,mad,qplus(1,j))
* On conserve en commentaire au cas ou...
*       d1 = conjg(alpha(j)) + un
*       d2 = icomp * (un - conjg(alpha(j)))
*       call CALVEC(d1,d2,cc,dim,mpm,mad,qmoins(1,j))
   10 continue
      end
************************************************************************
      subroutine CALVEC(d1,d2,cc,dim,mpm,mad,q)
      integer dim
      real cc(*)
      complex d1, d2, mpm(dim,*), mad(dim,*), q(dim)
*  Variables locales
      integer i,j, k
      real rnorm, u, rnormx
*       -----------------------------------------------------------
*       Calcul du vecteur colonne, de norme max, de Mad(alpha),
*       matrice des cofacteurs de M(alpha) determinee par d1 et d2.
*       Arguments d'entree  : d1,d2,cc,dim
*       Arguments de sortie : q
*       Tableaux de travail : mpm,mad
*        d1   : vaut 1+alpha
*        d2   : vaut i(alpha-1) pour M+ ou i(1-alpha) pour M-
*        cc   : coefficients de la matrice de rigidite du domaine
*               considere
*        dim  : dimension de l'espace
*        mpm,mad : stockage temporaire du symbole et de la matrice
*                  des cofacteurs
*        q    : vecteur recherche
*       -----------------------------------------------------------
*      Calcul du symbole M(alpha)
      call SYMBOL(d1,d2,cc,dim,mpm)
*      Calcul de la matrice des cofacteurs Mad(alpha)
      call MATCOF(dim,mpm,mad)
*      Determination de la colonne k de norme max de Mad(alpha)
      k = 1
      rnormx = 0.
      do 20 j=1,dim
       rnorm = 0.
       do 10 i=1,dim
        u = abs(mad(i,j))
   10   rnorm = rnorm + u*u
       if (rnorm .gt. rnormx) then
        rnormx = rnorm
        k = j
       endif
   20 continue
*      Recuperation du resultat et normalisation
*      (influence sensible sur la valeur du determinant)
      rnormx = sqrt(rnormx)
      do 30 i=1,dim
   30  q(i) = mad(i,k) / rnormx
c MODIF
c   30  q(i) = mad(i,1)
      end
************************************************************************
      subroutine EXSIEL(intrfa)
      character*(*) intrfa
*  Declarations generales et communs
      include 'singul.inc'
*  Variables locales
      integer nbfrsu, nbfix
      parameter (nbfrsu = 3, nbfix = 6)
      integer nbfiou, nbfrou, unlo(nbfix), codret
      integer action(2), axe(nbdomx),nudova, nbpado(2)
      integer multi, nbsep
      character*4 extsu(nbfrsu)
      character*6 etat
      character*200 nomfic(nbfix)
      real somrec(4),sommod(4), ang(3,nbdomx), ccp(nbcxx,nbdomx), parcou
      complex nu
* Initialisations
      data extsu /'.sgf','.m','.trf'/
      data unlo /nbfix*0/
*       -------------------------------------------------------------
*       Calcul des exposants de singularites d'un probleme elliptique
*       issu de l'elasticite sur un domaine possedant un coin en 2D
*       ou une arete en 3D.
*       Arguments d'entree :
*        intrfa : nom du fichier d'interface avec l'environnement
*                 contenant le nom du fichier de donnees et le nom
*                 du repertoire ou seront stockes les fichiers de
*                 resultat ; les noms sont attendus sous forme de
*                 chemins absolus.
*       -------------------------------------------------------------
*
*       Initialisation de la SD de gestion des messages en anglais
*       Le passage eventuel en francais ne se fera qu'apres l'ou-
*       verture du fichier de donnees.
      call INISTM(0,codret)
      if (codret .ne. 0) return
*
*       Gestion des fichiers.
*       Les unites logiques associes aux fichiers sont memorisees
*       dans le tableau unlo selon la correspondance suivante :
*        unlo(1) : fichier de donnees
*       Les fichiers suivants sont crees par le programme
*       - systematiquement
*        unlo(2) : fichier principal des resultats (.res)
*        unlo(3) : fichier contenant les racines calculees (.exp)
*       - selon l'action demandee, plus precisement si a1>1 (cf. LVACT)
*        unlo(4) : fichier contenant les valeurs des fonctions singu-
*                  lieres pour une exploitation graphique directe (.sgf)
*        unlo(5) : fichier contenant les scripts MATLAB permettant de
*                  calculer les fonctions singulieres (.m)
*        unlo(6) : fichier contenant les contraintes tangentielles (.trf)
      call GESFIC(intrfa,nbfix,unlo,nomfic,nbfiou)
      if (nbfiou .eq. 3) then
       call DELANG(unlo(1),codret)
       if (codret .ne. 0) goto 10
*
*       Lecture des donnees, verification et impression
*       initialisation de parametres et constantes.
       call LIVDON(unlo(1),unlo(2),dimx,nbdomx,nbcxx,prmin,dim,nbdom,
     & contx,cc,omega,cdlim,action,somrec,ang,axe,nudova,nbpado,codret)
       if (codret .ne. 0) goto 10
       if (action(2) .ge. 2) then
*        Ouverture des nbfrsu fichiers resultat supplementaires
*        necessaires dans ce cas. On sort s'ils ne sont pas tous ouverts.
        call OUVFIC(nomfic(1),nbfrsu,extsu,unlo(4),nomfic(4),nbfrou)
	if (nbfrou .lt. nbfrsu) goto 10
       endif
*
       dmeso = 2*dim
       dimanu = dmeso*nbdom
       call INIEPS(0)
       write(unlo(3),'(g23.15e3)') sseprc
       parcou = 0.
*
       if (action(1) .eq. 0) then
*
*          ACTION 0 : Recherche manuelle
*          ========
        call CALBAS(unlo(2),dim,nbcxx,nbdom,contx,cc,phi,cneu,alpha,
     &              qplus,codret)
        call RCMANU(unlo(2),nbpcer,toler,nu,multi)
        if (action(2).ge.1)
     &        call CALKER(unlo(2),nu,multi,action,parcou,codret)
*
       elseif (action(1) .eq. 1) then
*
*          ACTION 1 : Configuration fixe
*          ========
*   On imprime une valeur bidon pour respecter la structure du fichier
        write(unlo(3),*) parcou
        nbsep = 0
        call RCAUTO(unlo(2),action,parcou,somrec,sommod,nbsep,codret)
*
       elseif (action(1) .eq. 2) then
*
*          ACTION 2 : Configuration mobile autour d'un axe de rotation
*          ========
        call ROTREF(unlo(2),action,somrec,ang,axe,cc,ccp,nbcxx,dim,
     &              nbdom,codret)
*
       elseif (action(1) .eq. 3) then
*
*          ACTION 3 : Variation de la geometrie du domaine
*          ========
        call CALBAS(unlo(2),dim,nbcxx,nbdom,contx,cc,phi,cneu,alpha,
     &              qplus,codret)
        call VARGEO(unlo(2),action,prmin,somrec,nbpcer,nbprec,pimin,
     &         taimai,toler,nudova,ang,nbdom,omega,nbintx,absci,codret)
*
       elseif (action(1) .eq. 4) then
*
*          ACTION 4 : Orientation des materiaux
*          ========
        call ROTFIB(unlo(2),action,somrec,ang,axe,nbpado,cc,ccp,nbcxx,
     &              dim,nbdom,codret)
*
       elseif (action(1) .eq. 5) then
*
*          ACTION 5 : Suivi d'une arete courbe
*          ========
        call SUIVAR(unlo,action,somrec,cc,ccp,nbcxx,dim,nbdom,omega,
     &              codret)
*
       elseif (action(1) .eq. 6) then
*
*          ACTION 6 : Variation d'un coefficient Cij
*          ========
        call VARCO(unlo,action,somrec,contx,cc,nbcxx,dim,nbpado,codret)
*
       endif
      else
       call IMPMES(0,'EXSIEL',1,' ',codret)
       stop '===== EXSIEL : unexpected termination ====='
      endif
*
   10 close(unlo(1))
      call IMPMES(unlo(2),'EXSIEL',-2,' ',codret)
      do 20 nbfiou=2,nbfix
   20  if (unlo(nbfiou) .gt. 0) close(unlo(nbfiou))
      end
************************************************************************
      subroutine MAJTC(ius,coef,numco,dim,ctxdom,cc,codret)
      integer ius,numco,dim,ctxdom,codret
      real coef, cc(*)
*  Variables locales
      integer cmu(2:3), indlam
      real lambda, mu
*  Initialisation
      parameter (indlam = 2)
      data cmu /6, 21/
*       --------------------------------------------------------------
*       Mise a jour du tableau des coefficients des matrices de
*       rigidite dans le cas ou l'on fait varier l'un des coefficients.
*       Arguments d'entree  : ius,coef,numco,dim,ctxdom,cc
*       Arguments de sortie : cc,codret
*        ius    : unite logique du fichier de sortie
*        coef   : valeur du coefficient variable
*        numco  : indice dans le tableau commun (cc) du
*                 coefficient Cij destine a varier
*        dim    : dimension de l'espace
*        ctxdom : contexte du calcul du domaine considere
*        cc     : coefficients de la matrice de rigidite pour le
*                 domaine considere
*        codret : code de retour valant 0 si RAS, 1 si les valeurs des
*                 coefficients sont incompatibles dans le cas isotrope.
*       --------------------------------------------------------------
      codret = 0
      if (ctxdom .eq. 1) then
*
*       Elasticite isotrope
       if (numco .eq. indlam) then
        lambda = coef
        mu = cc(cmu(dim))
       else
        lambda = cc(indlam)
        mu = coef
       endif
       call CMRISO(ius,dim,lambda,mu,cc,codret)
*
      else
       if (dim.eq.2 .and. ctxdom.eq.2 .and. numco.eq.1) then
*        Elasticite orthotrope axisymetrique (C11=C22)
*        avec variation de C11 et C22
        cc(4) = coef
       endif
       cc(numco)=coef
      endif
      end
************************************************************************
      subroutine MATCOF(dim,mpm,mad)
      integer dim
      complex mpm(dim,*), mad(dim,*)
*  Variables locales
      integer i,j, i1,i2,j1,j2, iper(3)
*  Initialisation
      data iper /2,3,1/
*       -------------------------------------------------------
*       Calcul de la matrice des cofacteurs associee au symbole
*       M+(alpha) ou M-(alpha) en dimension 2 ou 3.
*       Arguments d'entree  : dim,mpm
*       Arguments de sortie : mad
*        dim  : dimension de l'espace
*        mpm  : symbole
*        mad  : matrice des cofacteurs recherchee
*       -------------------------------------------------------
      if (dim .eq. 2) then
       mad(1,1) =  mpm(2,2)
       mad(2,1) = -mpm(2,1)
       mad(1,2) = -mpm(1,2)
       mad(2,2) =  mpm(1,1)
      else
       do 10 j=1,dim
        j1 = iper(j)
        j2 = iper(j1)
        do 10 i=1,dim
         i1 = iper(i)
         i2 = iper(i1)
         mad(i,j) = mpm(i1,j1)*mpm(i2,j2) - mpm(i2,j1)*mpm(i1,j2)
   10  continue
      endif
      end
************************************************************************
      program PPSING
      call EXSIEL('INTERFACE')
      stop '===== End of computation ====='
      end
************************************************************************
      subroutine PR3PD2(a,b,c,r)
      real a(0:2), b(0:2), c(0:2), r(0:6)
*  Variables locales
      real d(0:4)
*       --------------------------------------------------------
*       Calcul du produit r des 3 polynomes de degre 2 a, b et c
*       Arguments d'entree  : a,b,c
*       Arguments de sortie : r
*        a,b,c : coefficients des polynomes de degre 2
*        r     : coefficients du polynome de degre 6
*                (r(i) coeff de x**i, i=0,6)
*       --------------------------------------------------------
      d(0) = b(0)*c(0)
      d(1) = b(0)*c(1) + b(1)*c(0)
      d(2) = b(0)*c(2) + b(1)*c(1) + b(2)*c(0)
      d(3) = b(1)*c(2) + b(2)*c(1)
      d(4) = b(2)*c(2)
*
      r(0) = a(0)*d(0)
      r(1) = a(0)*d(1) + a(1)*d(0)
      r(2) = a(0)*d(2) + a(1)*d(1) + a(2)*d(0)
      r(3) = a(0)*d(3) + a(1)*d(2) + a(2)*d(1)
      r(4) = a(0)*d(4) + a(1)*d(3) + a(2)*d(2)
      r(5) = a(1)*d(4) + a(2)*d(3)
      r(6) = a(2)*d(4)
      end
************************************************************************
      subroutine RCAUTO(unlo,action,parcou,somrec,sommod,nbsep,codret)
      integer unlo(*),action(2),nbsep,codret
      real parcou,somrec(4),sommod(4)
*  Declarations generales et communs
      include 'singul.inc'
*  Variables locales
      integer ius, numrac
*       ---------------------------------------------------------
*       Ensemble des etapes menant au calcul des exposants de
*       singularites, pour une configuration geometrique donnee.
*       Arguments d'entree  : unlo,action,parcou,somrec,nbsep
*       Arguments de sortie : sommod,nbsep,codret
*        unlo   : numeros des unites logiques des fichiers ou
*                 sont imprimes les resultats
*        action : caracterise l'action a realiser
*        parcou : parametre courant pilotant le calcul
*        somrec : rectangle (xmin,xmax,ymin,ymax) dans lequel
*                 s'effectue la recherche si action(1) >= 1
*        sommod : sommets du rectangle de recherche effectivement
*                 utilise (en principe il s'agit d'une deformation
*                 legere du rectangle initial somrec)
*        nbsep  : nombre de separateurs de racines de la
*                 configuration precedente (en entree), de celle
*                 qui vient d'etre calculee (en sortie)
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       ---------------------------------------------------------
      ius = unlo(1)
*
*       Calcul de la base de l'espace des solutions
*       -------------------------------------------
      call CALBAS(ius,dim,nbcxx,nbdom,contx,cc,phi,cneu,alpha,
     &            qplus,codret)
      if (codret .ne. 0) goto 10
*
*       Calcul des exposants de singularites
*       ------------------------------------
*        Recherche automatique dans un domaine rectangulaire
       call QUADRI(ius,prmin,somrec,nbpcer,nbprec,pimin,taimai,toler,
     & nbsep,nbintx,absci,numrac,sommod)
*
*       Impression des racines et poursuite eventuelle des actions
      call IMPRAC(unlo(2),numrac,codret)
      call SEPRAC(numrac,sommod,nbsep,absci)
      call ACSUIV(unlo,numrac,action,parcou,codret)
      call SEPARR(ius,codret)
*
   10 return
*
      end
************************************************************************
      subroutine RCMANU(ius,nbpcer,toler,nu,multi)
      integer ius,nbpcer,multi
      real toler
      complex nu
*  Variables locales
      integer irep, ks, nbsubx, nsubut, npcloc, codret
      parameter (ks = 2, nbsubx = 100)
      logical centcv
      real prnu, pinu, rc,rd, som(4)
      complex z0, intc(0:ks)
      character*52 txtsup
*  Points d'entree
      complex DETCAR
*       --------------------------------------------------------
*       Recherche manuelle de l'exposant de singularite nu.
*       Arguments d'entree  : ius,nbpcer,toler
*       Arguments de sortie : nu,multi
*        ius    : unite logique du fichier de sortie
*        nbpcer : nombre de points pour le calcul des
*                 integrales de contour sur un cercle
*        toler  : precision requise pour le calcul des exposants
*        nu     : exposant de singularite
*        multi  : multiplicite de nu
*       --------------------------------------------------------
      call IMPMES(ius,'RCMANU',-1,' ',codret)
      if (codret.ne.0) goto 85
      call IMPMES(  0,'RCMANU',-2,' ',codret)
*
   10 write(txtsup(1:12),'(i12)') nbpcer
      call IMPMES(  0,'RCMANU',-3,'-'//txtsup(1:12),codret)
      call IMPMES(  0,'RCMANU',-4,' ',codret)
      read(*,*,err=10) npcloc
      write(txtsup(1:12),'(i12)') npcloc
      call IMPMES(  0,'RCMANU',-5,'-'//txtsup(1:12),codret)
   30 continue
      centcv = .false.
      call IMPMES(  0,'RCMANU',-6,' ',codret)
      read(*,*,err=50) prnu, pinu, rc
      z0 = cmplx(prnu, pinu)
   40 continue
      call INCAUC(z0,rc,npcloc,ks,intc)
      rd = rc
      call DMULTI(intc(0),nbsubx,multi)
      nu = intc(1)
      if (multi .gt. 0) nu = nu / multi
      call IMPMES(  0,'RCMANU',-7,' ',codret)
      write(*,*) '==============> Nu = ', nu
      write(*,*) '            (I =     ', intc(1), ' )'
      write(*,*) ' '
      write(*,*) '                M  = ', multi, '   OK : ', multi.ge.0
      write(*,*) '            (I =     ', intc(0), ' )'
      write(*,*) ' '
      if (multi .gt. 0) write(*,*) '                D  = ', DETCAR(nu)
      write(*,*) ' '
CC
      if (multi .gt. 0) then
       call IMPMES(  0,'RCMANU',-8,' ',codret)
       som(1) = prnu-rd
       som(2) = prnu+rd
       som(3) = pinu-rd
       som(4) = pinu+rd
       nu = z0*multi
       call ESTIPR(nbpcer,toler,som,1,ks,multi,nu,rd,intc,nsubut,irep)
       nu = nu / multi
       write(*,*) '==============> Nu = ', nu
       write(*,*) '                r  = ', rd, '  codret = ',irep
       write(*,*) '                D  = ', DETCAR(nu)
      endif
CC
      if (centcv) then
       goto 60
      else
       goto 30
      endif
   50 call IMPMES(  0,'RCMANU',-9,' ',codret)
      read(*,*,err=50) irep
      if (irep .eq. 0) goto 80
      goto (30,60,70) irep
      goto 50
*
*       Le centre est conserve, on fait varier le rayon.
   60 centcv = .true.
      write(txtsup,90) nu
      call IMPMES(  0,'RCMANU',-10,'-'//txtsup,codret)
      call IMPMES(  0,'RCMANU',-11,' ',codret)
      read (*,*,err=60) rc
      if (rc .le. 0.) goto 50
      z0 = nu
      goto 40
*
*       Evaluation du determinant pour une valeur de nu donnee
   70 call IMPMES(  0,'RCMANU',-12,' ',codret)
      read(*,*,err=50) prnu, pinu
      z0 = cmplx(prnu, pinu)
      write(txtsup,90) DETCAR(z0)
      call IMPMES(  0,'RCMANU',-13,'-'//txtsup,codret)
      goto 50
*
   80 write(txtsup,90) nu
      call IMPMES(ius,'RCMANU',-14,'-'//txtsup,codret)
      if (codret.ne.0) goto 85
      write(txtsup(1:11),'(g11.4)') rd
      call IMPMES(ius,'RCMANU',-15,'-'//txtsup(1:11),codret)
      if (codret.ne.0) goto 85
      if (multi .gt. 1) then
       write(txtsup(1:12),'(i12)') multi
       call IMPMES(ius,'RCMANU',-16,'-'//txtsup(1:12),codret)
       if (codret.ne.0) goto 85
      endif
      write(ius,*,err=85)
   85 return
*
   90 format(1x,g23.15e3,1x,sp,g23.15e3,' *i')
      end
************************************************************************
      subroutine ROTFIB(unlo,action,somrec,ang,axe,nbpado,cc,ccp,nbcxx,
     &dim,nbdom,codret)
      integer nbdom,codret
      integer unlo(*),action(2),axe(nbdom),nbpado(2),nbcxx,dim
      real somrec(4), ang(3,nbdom), cc(nbcxx,nbdom),ccp(nbcxx,nbdom)
*  Variables locales
      integer i,j, axerot(3), nbsep, np, num
      real angrot(3), R(3,3), fconv, angrdg, sommod(4)
      character*16 txtsup
*  Initialisations
      parameter (fconv = 57.295779513082)
*       -------------------------------------------------------------
*       Ensemble des etapes menant au calcul des exposants de
*       singularites, pour une configuration geometrique fixe, mais
*       ou l'orientation des fibres des materiaux est variable, par
*       la prise en compte d'une rotation autour d'un des trois axes
*       du referenciel absolu.
*       La geometrie des domaines est invariante.
*       Module adapte a l'etude des materiaux composites.
*       Arguments d'entree  : unlo,action,somrec,
*                             ang,axe,nbpado,cc,nbcxx,dim,nbdom
*       Arguments de sortie : codret
*       Tableaux de travail : ccp
*        unlo   : numeros des unites logiques des fichiers ou
*                 sont imprimes les resultats
*        action : caracterise l'action a realiser
*        somrec : rectangle (xmin,xmax,ymin,ymax) dans lequel
*                 s'effectue la recherche si action(1) >= 1
*        ang    : tableau contenant l'angle de debut et l'increment de
*                 la rotation (en radians) pour chaque domaine
*        axe    : pour chaque domaine, numero de l'axe autour duquel
*                 se fait la rotation
*        nbpado : 1er elt  = nombre de pas a realiser (identique pour
*                  tous les materiaux)
*                 2eme elt = numero du domaine determinant l'angle a
*                  porter en abscisse des courbes
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*        ccp    : sauvegarde temporaire de cc
*        nbcxx  : nombre max de coefficients (1ere dimension de cc)
*        dim    : dimension de l'espace
*        nbdom  : nombre de domaines
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       -------------------------------------------------------------
*      Sauvegarde locale des coefficients Cij
      do 10 j=1,nbdom
       do 10 i=1,nbcxx
   10   ccp(i,j) = cc(i,j)
*
*       Les 2 dernieres rotations ne sont pas utilisees ici
      do 20 i=2,3
       angrot(i) = 0.
   20  axerot(i) = 1
*
      num = nbpado(2)
      nbsep = 0
      do 40 np=0,nbpado(1)-1
       call IMPMES(unlo(1),'ROTFIB',-1,' ',codret)
       do 30 i=1,nbdom
        write(txtsup(1:2),'(i2)') i
        call IMPMES(unlo(1),'ROTFIB',-2,'-'//txtsup(1:2),codret)
*        On tourne autour de l'axe axe(i) a partir de l'angle
*        ang(1,i) par pas de ang(2,i).
*        La rotation s'applique au repere : c'est le repere qui tourne.
*        Ici, on souhaite un repere fixe et des materiaux mobiles :
*        il faut donc inverser le sens de la rotation (signe -).
        angrot(1) = -(ang(1,i) + np*ang(2,i))
        axerot(1) = axe(i)
*
*        Pour le signe -, voir le commentaire precedent.
        angrdg = -angrot(1)*fconv
        write(txtsup,'(1x,f7.3,1x,f7.2)') -angrot(1), angrdg
        call IMPMES(unlo(1),'ROTFIB',-3,'-'//txtsup,codret)
        if (codret .ne. 0) goto 50
        call IMPMES(      0,'ROTFIB',-3,'-'//txtsup,codret)
        if (codret .ne. 0) goto 50
*      Calcul de la matrice de rotation R correspondante
        call MATROT(angrot,axerot,R)
*      Calcul des nouveaux coefficients cc
        call CALCCP(ccp(1,i),R,cc(1,i))
        call IMPMES(unlo(1),'ROTFIB',-4,' ',codret)
        if (codret .ne. 0) goto 50
        call ECRCOF(unlo(1),dim,cc(1,i),codret)
        if (codret .ne. 0) goto 50
   30  continue
*       Calcul des exposants de singularites
*       pour cette orientation des materiaux
       angrdg = (ang(1,num)+np*ang(2,num)) * fconv
       write(unlo(2),'(g23.15e3)',err=50) angrdg 
       call RCAUTO(unlo,action,angrdg,somrec,sommod,nbsep,codret)
   40 continue
*
   50 return
      end
************************************************************************
      subroutine ROTREF(unlo,action,somrec,ang,axe,cc,ccp,nbcxx,dim,
     &nbdom,codret)
      integer unlo(*),action(2),axe,nbcxx,dim,nbdom,codret
      real somrec(4), ang(3), cc(nbcxx,nbdom),ccp(nbcxx,nbdom)
*  Variables locales
      integer i,j, axerot(3), nbcal, nbsep
      real angrot(3), R(3,3), fconv, angrdg, sommod(4),siginc
      character*16 txtsup
*  Initialisations
      parameter (fconv = 57.295779513082)
*       -------------------------------------------------------------
*       Ensemble des etapes menant au calcul des exposants de
*       singularites, pour une configuration mobile par la prise en
*       compte d'une rotation du referentiel local autour d'un des
*       trois axes du referenciel absolu.
*       La geometrie des domaines est invariante.
*       Arguments d'entree  : unlo,action,somrec,
*                             ang,axe,cc,nbcxx,dim,nbdom
*       Arguments de sortie : codret
*       Tableaux de travail : ccp
*        unlo   : numeros des unites logiques des fichiers ou
*                 sont imprimes les resultats
*        action : caracterise l'action a realiser
*        somrec : rectangle (xmin,xmax,ymin,ymax) dans lequel
*                 s'effectue la recherche si action(1) >= 1
*        ang    : tableau contenant l'angle de debut, de fin et
*                 l'increment de la rotation (en radians)
*        axe    : numero de l'axe autour duquel se fait la rotation
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*        ccp    : sauvegarde temporaire de cc
*        nbcxx  : nombre max de coefficients (1ere dimension de cc)
*        dim    : dimension de l'espace
*        nbdom  : nombre de domaines
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       -------------------------------------------------------------
*      Sauvegarde locale des coefficients Cij
      do 10 j=1,nbdom
       do 10 i=1,nbcxx
   10   ccp(i,j) = cc(i,j)
*
*      On tourne autour de l'axe donne en entree entre les angles
*      ang(1) et ang(2) par pas de ang(3).
      angrot(1) = ang(1)
      axerot(1) = axe
*       Les 2 dernieres rotations ne sont pas utilisees ici
      do 20 i=2,3
       angrot(i) = 0.
   20  axerot(i) = 1
*
      siginc = sign(1.,ang(3))
      nbsep = 0
      nbcal = 0
   30 angrot(1) = ang(1) + nbcal*ang(3)
      if (siginc*angrot(1) .gt. siginc*ang(2)) goto 50
      angrdg = angrot(1)*fconv
      write(txtsup,'(1x,f7.3,1x,f7.2)') angrot(1), angrdg
      call IMPMES(unlo(1),'ROTREF',-1,'-'//txtsup,codret)
      if (codret .ne. 0) goto 50
      call IMPMES(      0,'ROTREF',-1,'-'//txtsup,codret)
      if (codret .ne. 0) goto 50
*      Calcul de la matrice de rotation R correspondante
      call MATROT(angrot,axerot,R)
*      Calcul des nouveaux coefficients cc
      do 40 i=1,nbdom
       call CALCCP(ccp(1,i),R,cc(1,i))
       write(txtsup(1:2),'(i2)') i
       call IMPMES(unlo(1),'ROTREF',-2,'-'//txtsup(1:2),codret)
       if (codret .ne. 0) goto 50
       call ECRCOF(unlo(1),dim,cc(1,i),codret)
       if (codret .ne. 0) goto 50
   40  continue
*       Calcul des exposants de singularites
*       pour cette configuration geometrique
      write(unlo(2),'(g23.15e3)',err=50) angrdg
      call RCAUTO(unlo,action,angrdg,somrec,sommod,nbsep,codret)
      nbcal = nbcal + 1
      goto 30
*
   50 return
      end
************************************************************************
      subroutine SUIVAR(unlo,action,somrec,cc,ccp,nbcxx,dim,nbdom,omega,
     &codret)
      integer unlo(*),action(2),nbcxx,dim,nbdom,codret
      real somrec(4), cc(nbcxx,nbdom),ccp(nbcxx,nbdom)
      real omega(0:nbdom)
*  Variables locales
      integer i,j, nbsep, typ, ival, nb, codlec
      real R(3,3), fconv, sommod(4),valcs, parcou
      character*12 txtsup
*  Initialisations
      parameter (typ=2)
*       -------------------------------------------------------------
*       Ensemble des etapes menant au calcul des exposants de
*       singularites, pour une configuration mobile le long d'une
*       arete courbe.
*       Les donnees necessaires sont, pour chaque point de calcul, la
*       matrice des composantes de la base locale dans la base absolue,
*       les angles d'ouverture des domaines dans la base locale.
*       Arguments d'entree  : unlo,action,somrec,cc,nbcxx,dim,nbdom
*       Arguments de sortie : cc,omega,codret
*       Tableaux de travail : ccp
*        unlo   : numeros des unites logiques du fichier de donnees et
*                 des fichiers ou sont imprimes les resultats
*        action : caracterise l'action a realiser
*        somrec : rectangle (xmin,xmax,ymin,ymax) dans lequel
*                 s'effectue la recherche si action(1) >= 1
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*                 --> modifie au cours du calcul de facon a mettre
*                     a jour le commun blanc, donc modifie en sortie
*        ccp    : sauvegarde temporaire de cc
*        nbcxx  : nombre max de coefficients (1ere dimension de cc)
*        dim    : dimension de l'espace
*        nbdom  : nombre de domaines
*        omega  : angles definissant la geometrie des domaines
*                 --> modifie au cours du calcul de facon a mettre
*                     a jour le commun blanc, donc modifie en sortie
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       -------------------------------------------------------------
      fconv = atan(1.)/45.
*      Sauvegarde locale des coefficients Cij
      do 10 j=1,nbdom
       do 10 i=1,nbcxx
   10   ccp(i,j) = cc(i,j)
*
      nbsep = 0
*      Lecture du parametre courant, de la matrice, des angles :
   20 nb = 1
      call LECVAL(unlo(1),nb,typ,ival,parcou,codlec)
      if (codlec .eq. 2) goto 70
      if (codlec .ne. 0) goto 80
      nb = 9
      call LECVAL(unlo(1),nb,typ,ival,R,codlec)
      if (codlec .ne. 0) goto 80
*      La matrice est lue en ligne et stockee en colonne :
*      il faut transposer.
      do 30 i=1,2
       do 30 j=i+1,3
        valcs = R(i,j)
        R(i,j) = R(j,i)
   30   R(j,i) = valcs
      nb = nbdom+1
      call LECVAL(unlo(1),nb,typ,ival,omega(0),codlec)
      if (codlec .ne. 0) goto 80
*       Conversion en radian
      do 40 i=0,nbdom
   40  omega(i) = omega(i)*fconv
*
      write(txtsup,'(1x,g11.4)') parcou
      call IMPMES(unlo(2),'SUIVAR',-1,'-'//txtsup,codret)
      if (codret .ne. 0) goto 70
      call IMPMES(      0,'SUIVAR',-1,'-'//txtsup,codret)
      if (codret .ne. 0) goto 70
      call IMPMES(unlo(2),'SUIVAR',-2,' ',codret)
      if (codret .ne. 0) goto 70
      do 50 i=1,3
   50  write(unlo(2),'(3(1x,g13.6))',err=70) (R(i,j),j=1,3)
      call IMPMES(unlo(2),'SUIVAR',-3,' ',codret)
      if (codret .ne. 0) goto 70
      write(unlo(2),'(6(1x,g13.6))',err=70) (omega(i),i=0,nbdom)
*
*      Calcul des nouveaux coefficients cc
      do 60 i=1,nbdom
       call CALCCP(ccp(1,i),R,cc(1,i))
       write(txtsup(1:2),'(i2)') i
       call IMPMES(unlo(2),'SUIVAR',-4,'-'//txtsup(1:2),codret)
       if (codret .ne. 0) goto 70
       call ECRCOF(unlo(2),dim,cc(1,i),codret)
       if (codret .ne. 0) goto 70
   60  continue
*      Calcul des exposants de singularites
*      pour cette configuration geometrique
      write(unlo(3),'(g23.15e3)',err=70) parcou
      call RCAUTO(unlo(2),action,parcou,somrec,sommod,nbsep,codret)
      goto 20
*
   70 return
   80 call IMPMES(unlo(2),'SUIVAR',5,' ',codret)
      call IMPMES(      0,'SUIVAR',5,' ',codret)
      codret = 1
      end
************************************************************************
      subroutine SYMBOL(d1,d2,cc,dim,mpm)
      integer dim
      complex d1,d2,mpm(dim,*)
      real cc(*)
*  Variables locales
      complex d12,d22,d1d2
*       -------------------------------------------------------------
*       Calcul du symbole M+(alpha) ou M-(alpha) en dimension 2 ou 3.
*       Arguments d'entree  : d1,d2,cc,dim
*       Arguments de sortie : mpm
*        d1   : vaut 1+alpha
*        d2   : vaut i(alpha-1) pour M+ ou i(1-alpha) pour M-
*        cc   : coefficients de la matrice de rigidite
*        dim  : dimension de l'espace du probleme
*        mpm  : symbole calcule
*       -------------------------------------------------------------
      d12 = d1*d1
      d22 = d2*d2
      d1d2 = d1*d2
      if (dim .eq. 2) then
       mpm(1,1) = cc(1)*d12 + 2.*cc(3)*d1d2 + cc(6)*d22
       mpm(2,1) = cc(3)*d12 + (cc(2)+cc(6))*d1d2 + cc(5)*d22
       mpm(1,2) = mpm(2,1)
       mpm(2,2) = cc(6)*d12 + 2.*cc(5)*d1d2 + cc(4)*d22
      else
       mpm(1,1) = cc(1)*d12 + 2.*cc(6)*d1d2 + cc(21)*d22
       mpm(2,1) = cc(6)*d12 + (cc(2)+cc(21))*d1d2 + cc(11)*d22
       mpm(3,1) = cc(5)*d12 + (cc(4)+cc(20))*d1d2 + cc(18)*d22
       mpm(1,2) = mpm(2,1)
       mpm(2,2) = cc(21)*d12 + 2.*cc(11)*d1d2 + cc(7)*d22
       mpm(3,2) = cc(20)*d12 + (cc(10)+cc(18))*d1d2 + cc(9)*d22
       mpm(1,3) = mpm(3,1)
       mpm(2,3) = mpm(3,2)
       mpm(3,3) = cc(19)*d12 + 2.*cc(17)*d1d2 + cc(16)*d22
      endif
      end
************************************************************************
      subroutine VARCO(unlo,action,somrec,contx,cc,nbcxx,dim,nucodo,
     &codret)
      integer unlo(*),action(2),contx(*),nbcxx,dim,nucodo(2),codret
      real somrec(4), cc(nbcxx,*)
*  Variables locales
      integer nbsep, typ, ival, nb, codlec, ius, numdom
      real sommod(4), parcou
      logical defpos
      character*12 txtsup
*  Initialisations
      parameter (typ=2, nb=1)
*       --------------------------------------------------------------
*       Ensemble des etapes menant au calcul des exposants de
*       singularites, pour une configuration geometrique fixe, mais
*       ou l'on fait varier un coefficient de la matrice de rigidite
*       dans un domaine.
*       Arguments d'entree  : unlo,action,somrec,contx,cc,nbcxx,nucodo
*       Arguments de sortie : codret
*        unlo   : numeros des unites logiques du fichier de donnees et
*                 des fichiers ou sont imprimes les resultats
*        action : caracterise l'action a realiser
*        somrec : rectangle (xmin,xmax,ymin,ymax) dans lequel
*                 s'effectue la recherche si action(1) >= 1
*        contx  : contexte du calcul pour chaque domaine
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*                 --> modifie au cours du calcul de facon a mettre
*                     a jour le commun blanc, donc modifie en sortie
*        nbcxx  : nombre max de coefficients (1ere dimension de cc)
*        dim    : dimension de l'espace du probleme
*        nucodo : 1er elt  = indice dans le tableau commun (cc) du
*                 coefficient Cij destine a varier.
*                 2eme elt = numero du domaine ou les caracteristiques
*                 du materiau sont destinees a varier
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       --------------------------------------------------------------
      ius = unlo(2)
*
      numdom = nucodo(2)
      nbsep = 0
*      Lecture de la valeur du coefficient
   10 call LECVAL(unlo(1),nb,typ,ival,parcou,codlec)
      if (codlec .eq. 2) goto 70
      if (codlec .ne. 0) goto 80
*
*      Impression dans le fichier resultat pour le suivi du calcul
      write(txtsup,'(g12.5)') parcou
      call IMPMES(ius,'VARCO',-1,'-'//txtsup,codret)
      if (codret .ne. 0) goto 70
      call IMPMES(      0,'VARCO',-1,'-'//txtsup,codret)
      if (codret .ne. 0) goto 70
*
*      Mise a jour du tableau des coefficients
      call MAJTC(ius,parcou,nucodo(1),dim,contx(numdom),cc(1,numdom),
     & codret)
      if (codret.ne.0) then
       call IMPMES(ius,'VARCO',4,' ',codret)
       call IMPMES(  0,'VARCO',4,' ',codret)
       goto 10
      endif
      call IMPMES(ius,'VARCO',-5,' ',codret)
      call ECRCOF(ius,dim,cc(1,numdom),codret)
*
*       La matrice des coefficients doit etre definie positive
      call TESPOS(dim,cc(1,numdom),defpos,codret)
      if (.not. defpos) then
       call IMPMES(ius,'VARCO',3,' ',codret)
       call IMPMES(  0,'VARCO',3,' ',codret)
       call IMPMES(ius,'VARCO',4,' ',codret)
       call IMPMES(  0,'VARCO',4,' ',codret)
       goto 10
      endif
*
*      Calcul des exposants de singularites
      write(unlo(3),'(g23.15e3)',err=70) parcou
      call RCAUTO(unlo(2),action,parcou,somrec,sommod,nbsep,codret)
      goto 10
*
   70 return
   80 call IMPMES(ius,'VARCO',2,' ',codret)
      call IMPMES(  0,'VARCO',2,' ',codret)
      codret = 1
      end
************************************************************************
      subroutine VARGEO(unlo,action,prmin,somrec,nbpcer,nbprec,pimin,
     & taimai,toler,nufron,angvar,nbdom,omega,nbintx,absci,codret)
      integer unlo(*),action(2),nbpcer,nbprec,nufron,nbdom,nbintx
      integer codret
      real prmin, somrec(4),pimin,taimai,toler, angvar(3),omega(0:nbdom)
      real absci(nbintx)
      character*16 txtsup
*  Variables locales
      integer typ, ival, nbcal, nbsep, numrac
      real fconv, angrdg,sommod(4),siginc
*  Initialisations
      parameter (typ = 2, fconv = 57.295779513082)
*       --------------------------------------------------------------
*       Ensemble des etapes menant au calcul des exposants de
*       singularites, pour une configuration geometrique mobile par la
*       prise en compte d'une variation angulaire de la position d'une
*       frontiere externe ou entre deux domaines contigus.
*       Arguments d'entree  : unlo,action,prmin,somrec,nbpcer,nbprec,
*                             pimin,taimai,toler,nufron,angvar,nbdom,
*                             omega,nbintx
*       Arguments de sortie : omega,codret
*       Tableaux de travail : absci
*        unlo   : numeros des unites logiques des fichiers ou
*                 sont imprimes les resultats
*        action : caracterise l'action a realiser
*        prmin  : valeur min de l'abscisse definissant le rectangle
*                 pour la recherche automatique de racines
*        somrec : rectangle (xmin,xmax,ymin,ymax) dans lequel
*                 s'effectue la recherche si action(1) >= 1
*        nbpcer,nbprec : nombre de points pour le calcul des
*                      integrales sur un cercle et sur un rectangle
*        pimin  : partie imaginaire min en deca de laquelle les
*                 exposants ne sont pas recherches
*        taimai : taille min de la maille rectangulaire lors du
*                 decoupage du plan pour la recherche des racines
*        toler  : precision requise pour le calcul des exposants
*        nufron : numero de la frontiere variable
*        angvar : tableau contenant l'angle de debut, de fin et
*                 l'increment de la variation angulaire (en radians)
*        nbdom  : nombre de domaines
*        omega  : angles definissant la geometrie des domaines
*                 --> modifie au cours du calcul de facon a mettre
*                     a jour le commun blanc, donc en sortie
*        nbintx : nombre max de separateurs de zones permis
*        absci  : abscisses des separateurs de zones contenant des
*                 racines, determinees lors de la phase d'analyse
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       --------------------------------------------------------------
      siginc = sign(1.,angvar(3))
      nbsep = 0
      nbcal = 0
   30 omega(nufron) = angvar(1) + nbcal*angvar(3)
      if (siginc*omega(nufron) .gt. siginc*angvar(2)) goto 50
      angrdg = omega(nufron)*fconv
      write(txtsup,'(1x,f7.3,1x,f7.2)') omega(nufron), angrdg
      call IMPMES(unlo(1),'VARGEO',-1,'-'//txtsup,codret)
      if (codret.ne.0) goto 50
      call IMPMES(      0,'VARGEO',-1,'-'//txtsup,codret)
      if (codret.ne.0) goto 50
*      Modification de la geometrie
*      (modification du tableau omega, donc du commun blanc
*       general reference au moment du calcul de la matrice A(nu))
      call IMPMES(unlo(1),'VARGEO',-2,' ',codret)
      call ECRVAL(unlo(1),nbdom+1,typ,ival,omega(0),codret)
*      Calcul des exposants de singularites
*      pour cette configuration geometrique
      write(unlo(2),'(g23.15e3)',err=50) angrdg
      call QUADRI(unlo(1),prmin,somrec,nbpcer,nbprec,pimin,taimai,toler,
     & nbsep,nbintx,absci,numrac,sommod)
      call IMPRAC(unlo(2),numrac,codret)
      call SEPRAC(numrac,sommod,nbsep,absci)
      call ACSUIV(unlo,numrac,action,angrdg,codret)
      call SEPARR(unlo(1),codret)
      if (codret .ne. 0) goto 50
      nbcal = nbcal + 1
      goto 30
*
   50 return
      end
**************************** Fin du fichier ****************************
