function [nbrac,xr,racr,xc,prracc,piracc,codret] = lecrac(lisfic,prmax)
% [nbrac,xr,racr,xc,prracc,piracc,codret] = lecrac(lisfic,prmax)
%
% Lecture d'un ou plusieurs fichier(s) contenant les exposants
% et mise en place de la structure de donnees
% ENTREE
%  lisfic : liste des noms de fichiers a exploiter
%           (un nom par ligne de la matrice)
%  prmax  : partie reelle max a prendre en compte (les racines de partie
%           reelle > prmax sont eliminees)
% SORTIE
%  nbrac  : nombre de racines
%  racr   : racines reelles
%  xr     : abscisses correspondantes
%
%  prracc : parties reelles et
%  piracc :         imaginaires des racines complexes
%  xc     : abscisses correspondantes
%
%  codret : code de retour valant 0 si RAS, 1 en cas d'erreur
%
codret = 0;
%
xr = []; racr = [];
xc = []; prracc = []; piracc = [];
wn = []; z = [];
% Nombre de colonnes du fichier de donnees :
nbcd = 3;
%
%     1. Lecture du fichier de donnees (xxx.exp)
%        ---------------------------------------
for nomfic=lisfic'
 nomfic=sanesp(nomfic');
 [fid, message] = fopen(nomfic);
 if (fid < 0)
  disp([message,' (file ''',nomfic,''')']), codret = 1; return
 end
 disp(['Reading file ',nomfic]);
 [epsilon nbelu] = fscanf(fid,'%g',1);
 [absci nbelu] = fscanf(fid,'%g',1);
 while nbelu ~= 0
  nbrac = fscanf(fid,'%d',1);
  if (nbrac > 0)
   [rac nb] = fscanf(fid,'%g',[nbcd nbrac]);
   if (nb ~= nbcd*nbrac)
    disp('*** Error : incomplete data file.'), codret = 1; return
   end
%   Prise en compte de la limite sur la partie reelle
   ind = find(rac(1,:) < prmax);
   nbrac = length(ind);
   wn = [wn [absci ; nbrac]];
   z = [z rac(1:2,ind)];
  end
  [absci nbelu] = fscanf(fid,'%g',1);
 end
 fclose(fid);
end
%
if isempty(wn)
 disp('*** No exponent found.')
 codret = 1; return
else
 nbrac = sum(wn(2,:));
end
%
%     2. Separation des racines reelles des racines complexes
%        ----------------------------------------------------
%
% Une racine est declaree reelle si sa partie imaginaire est < epsilon.
kr=0;
kc=0;
i=0;
for absci=1:size(wn,2)
 for j=1:wn(2,absci)
  i=i+1;
  if abs(z(2,i)) < epsilon
%   Racine relle
   kr=kr+1;
   racr(kr)=z(1,i);
   xr(kr)=wn(1,absci);
  else
%   Racine complexe
   kc=kc+1;
   prracc(kc)=z(1,i);
   piracc(kc)=z(2,i);
   xc(kc)=wn(1,absci);
  end
 end
end
%
%     3. Verifications et commentaires
%        -----------------------------
%
disp('========')
if (nbrac == i)
 disp(['Number of exponents : ',int2str(i)])
else
 disp(['*** Error on the number of exponents : ',int2str(i),...
      ' counted up versus ',int2str(nbrac),' data.'])
 nbrac = i;
end
%if (nbrac <= 0), codret = 1; return, end
%
ind = find(abs(z(2,:)) > epsilon);
disp(['The separation threshold between real and complex exponents',...
' is ',num2str(epsilon),'.'])
if isempty(ind)
 disp('All the exponents are real.')
else
 disp(['The imaginary parts are comprised between ',...
 num2str(min(abs(z(2,ind)))),' and ',num2str(max(abs(z(2,ind))))])
end
disp('========')
